{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Assert;

interface

uses
  System.SysUtils, WEBLib.Utils;

type
  EAssert = class(Exception);

  Assert = record
  strict private
    class procedure FormatAreEqualMessage(const ACondition: Boolean; const AExpected, AActual, AMessage: string); static;
  public
    class procedure AreEqual(const AExpected, AActual: Integer; const AMessage: string = string.Empty); overload; static;
    class procedure AreEqual(const AExpected, AActual: string; const AMessage: string = string.Empty); overload; static;
    class procedure AreEqual(const AExpected, AActual: TDateTime; const AMessage: string = string.Empty); overload; static;
    class procedure IsFalse(const ACondition: Boolean; const AMessage: string = string.Empty); static;
    class procedure IsTrue(const ACondition: Boolean; const AMessage: string = string.Empty); static;
  end;

implementation

uses
  System.DateUtils, browserconsole;

resourcestring
  SEqualStr = 'Expected %s is equal to actual %s. [%s]';
  SFailed = 'Failed: ';
  SIsFalse = 'Condition is False when False expected. [%s]';
  SIsTrue = 'Condition is True when True expected. [%s]';
  SIsFalseError = 'Condition is False when False expected. [%s]';
  SIsTrueError = 'Condition is False when True expected. [%s]';
  SNotEqualErrorStr = 'Expected %s is not equal to actual %s. [%s]';
  SSuccess = 'Success: ';

{ Assert }

class procedure Assert.AreEqual(const AExpected, AActual: Integer; const AMessage: string);
begin
  FormatAreEqualMessage(AExpected = AActual, AExpected.ToString, AActual.ToString, AMessage);
end;

class procedure Assert.AreEqual(const AExpected, AActual, AMessage: string);
begin
  FormatAreEqualMessage(AExpected = AActual, AExpected, AActual, AMessage);
end;

class procedure Assert.AreEqual(const AExpected, AActual: TDateTime; const AMessage: string);
begin
  FormatAreEqualMessage(SameDateTime(AExpected, AActual), AExpected.ToString, AActual.ToString, AMessage);
end;

class procedure Assert.FormatAreEqualMessage(const ACondition: Boolean; const AExpected, AActual, AMessage: string);
begin
  if ACondition then
    Writeln(SSuccess + string.Format(SEqualStr, [AExpected, AActual, AMessage]))
  else
    Writeln(SFailed + string.Format(SNotEqualErrorStr, [AExpected, AActual, AMessage]));
end;

class procedure Assert.IsFalse(const ACondition: Boolean; const AMessage: string);
begin
  if ACondition then
    Writeln(SFailed + string.Format(SIsFalseError, [AMessage]))
  else
    Writeln(SSuccess + string.Format(SIsFalse, [AMessage]));
end;

class procedure Assert.IsTrue(const ACondition: Boolean; const AMessage: string);
begin
  if not ACondition then
    Writeln(SFailed + string.Format(SIsTrueError, [AMessage]))
  else
    Writeln(SSuccess + string.Format(SIsTrue, [AMessage]));
end;

end.
