{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ContinuousScroll;

interface

uses
  {$IFNDEF LCLTMSWEB}
  Classes, Web, JS, WEBLib.Controls, Vcl.Controls, WEBLib.JSON, Vcl.Graphics,
  WEBLib.Graphics, Types, WEBLib.HTML;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Web, JS, WEBLib.Controls, WEBLib.JSON, Controls, Graphics,
  WEBLib.Graphics, Types, WEBLib.HTML;
  {$ENDIF}

type
  TContinuousScrollFetchPageEvent = procedure(Sender: TObject; APageSize: Integer; APageNumber: Integer; var AURL: string) of object;
  TContinuousScrollJSONToItemEvent = procedure(Sender: TObject; AObject: TJSONObject; var AHTML: string) of object;
  TContinuousScrollObjectToArrayEvent = procedure(Sender: TObject; AObject: TJSObjectRecord; var AArray: TJSArrayRecord) of object;
  TContinuousScrollResponseToArrayEvent = procedure(Sender: TObject; AResponse: string; var AArray: TJSArrayRecord) of object;
  TContinuousScrollGetDataEvent = procedure(Sender: TObject; var AArray: TJSArrayRecord; var ALoadMoreData: Boolean) of object;
  TContinuousScrollGetItemEvent = procedure(Sender: TObject; AElementControl: TControl; AObject: TJSONObject; AIndex: Integer; AElement: TJSHTMLElementRecord) of object;
  TContinuousScrollItemFieldEvent = procedure(Sender: TObject; AIndex: Integer; AFieldName: string; var AValue: string) of object;
  TContinuousScrollPageLoadedEvent = procedure(Sender: TObject; APageNumber: Integer) of object;

  TContinuousScrollLoadType = (ltButton, ltScroll, ltNever);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebContinuousScroll = class(TWebCustomControl)
  private
    FOnFetchNextPage: TContinuousScrollFetchPageEvent;
    FPageNumber: Integer;
    FPageSize: Integer;
    FLoadType: TContinuousScrollLoadType;
    FOnJSONToItem: TContinuousScrollJSONToItemEvent;
    FOnObjectToArray: TContinuousScrollObjectToArrayEvent;
    FShowLoading: Boolean;
    FElementListEndClassName: TElementClassName;
    FListEndText: string;
    FElementLoadClassName: TElementClassName;
    FShowEnd: Boolean;
    FItemTemplate: string;
    FLoadScrollPercent: Integer;
    FElementButtonClassName: TElementClassName;
    FButtonText: string;
    FOnGetListItem: TContinuousScrollGetItemEvent;
    FOnItemGetFieldValue: TContinuousScrollItemFieldEvent;
    FOnResponseToArray: TContinuousScrollResponseToArrayEvent;
    FOnPageLoaded: TContinuousScrollPageLoadedEvent;
    FOnGetData: TContinuousScrollGetDataEvent;
    FAutoLoadFirstPage: Boolean;
    FHeaders: TStringList;
    function IsButtonTextStored: Boolean;
    function IsListEndTextStored: Boolean;
    procedure SetItemTemplate(const Value: string);
    procedure SetHeaders(const Value: TStringList);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    procedure FetchNextPage; virtual;
    function GetChildElement(AChildIndex: Integer): TJSHTMLElement;
    function GetChildElements: TJSNodeList;
  published
    property Align;
    property Anchors;
    property AutoLoadFirstPage: Boolean read FAutoLoadFirstPage write FAutoLoadFirstPage default True;
    property ShowHint;
    property Visible;
    property TabOrder;
    property TabStop;
    property Headers: TStringList read FHeaders write SetHeaders;
    property ButtonText: string read FButtonText write FButtonText stored IsButtonTextStored;
    property ListEndText: string read FListEndText write FListEndText stored IsListEndTextStored;
    property ElementPosition;
    property ElementFont;
    property ElementButtonClassName: TElementClassName read FElementButtonClassName write FElementButtonClassName;
    property ElementLoadClassName: TElementClassName read FElementLoadClassName write FElementLoadClassName;
    property ElementListEndClassName: TElementClassName read FElementListEndClassName write FElementListEndClassName;
    property ItemTemplate: string read FItemTemplate write SetItemTemplate;
    property LoadType: TContinuousScrollLoadType read FLoadType write FLoadType default ltButton;
    property PageSize: Integer read FPageSize write FPageSize default 10;
    property PageNumber: Integer read FPageNumber write FPageNumber default 1;
    property LoadScrollPercent: Integer read FLoadScrollPercent write FLoadScrollPercent default 90;
    property ShowLoading: Boolean read FShowLoading write FShowLoading default True;
    property ShowEnd: Boolean read FShowEnd write FShowEnd default True;
    property OnFetchNextPage: TContinuousScrollFetchPageEvent read FOnFetchNextPage write FOnFetchNextPage;
    property OnObjectToArray: TContinuousScrollObjectToArrayEvent read FOnObjectToArray write FOnObjectToArray;
    property OnResponseToArray: TContinuousScrollResponseToArrayEvent read FOnResponseToArray write FOnResponseToArray;
    property OnJSONToItem: TContinuousScrollJSONToItemEvent read FOnJSONToItem write FOnJSONToItem;
    property OnGetListItem: TContinuousScrollGetItemEvent read FOnGetListItem write FOnGetListItem;
    property OnItemGetFieldValue: TContinuousScrollItemFieldEvent read FOnItemGetFieldValue write FOnItemGetFieldValue;
    property OnPageLoaded: TContinuousScrollPageLoadedEvent read FOnPageLoaded write FOnPageLoaded;
    property OnGetData: TContinuousScrollGetDataEvent read FOnGetData write FOnGetData;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

{ TWebContinuousScroll }

procedure TWebContinuousScroll.Clear;
begin
//
end;

constructor TWebContinuousScroll.Create(AOwner: TComponent);
begin
  inherited;
  Height := 300;
  Width := 250;
  FAutoLoadFirstPage := True;
  FLoadType := ltButton;
  FLoadScrollPercent := 90;
  FPageSize := 10;
  FPageNumber := 1;
  FButtonText := 'Show more';
  FListEndText := 'End of results';
  FShowLoading := True;
  FShowEnd := True;
  FHeaders := TStringList.Create;
end;

destructor TWebContinuousScroll.Destroy;
begin
  FHeaders.Free;
  inherited;
end;

procedure TWebContinuousScroll.FetchNextPage;
begin

end;

function TWebContinuousScroll.GetChildElement(
  AChildIndex: Integer): TJSHTMLElement;
begin
  Result := nil;
end;

function TWebContinuousScroll.GetChildElements: TJSNodeList;
begin
  Result := nil;
end;

function TWebContinuousScroll.IsButtonTextStored: Boolean;
begin
  Result := FButtonText <> 'Show more';
end;

function TWebContinuousScroll.IsListEndTextStored: Boolean;
begin
  Result := FListEndText <> 'End of results';
end;

procedure TWebContinuousScroll.Paint;
var
  r: TRect;
  av,sv,fa,ah,cid,cv,ct: string;
  xs,ys,ml,hl: integer;
  hr,cr: TRect;
  I: Integer;
  itemh: Integer;
begin
  inherited;
  itemh := 70;
  Canvas.Pen.Color := clGray;
  Canvas.Pen.Style := psDot;
  r := Rect(0, 0, Width, itemh);
  for I := 0 to (Height div itemh) do
  begin
    Canvas.Rectangle(r);
    if (ItemTemplate <> '') then
    begin
      HtmlDrawEx(Canvas, ItemTemplate, r, nil, -1,-1,-1,-1,-1,
        false,false,false,false,false,false,true,false,'',1.0,clBlue,clNone,clNone,clGray,av,sv,fa,ah,xs,ys,hl,ml,hr,cr,cid,cv,ct);
    end;
    r.Top := r.Bottom - 1;
    r.Bottom := r.Top + itemh;
  end;

end;

procedure TWebContinuousScroll.SetHeaders(const Value: TStringList);
begin
  FHeaders.Assign(Value);
end;

procedure TWebContinuousScroll.SetItemTemplate(const Value: string);
begin
  FItemTemplate := Value;
  Invalidate;
end;

end.
