{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ElectronPGCDS;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, DB, DBClient, WEBLib.Controls, Web, jsdelphisystem, js, WEBLib.CDS,
  WEBLib.ElectronCDS;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, DB, WEBLib.Controls, Web, jsdelphisystem, WEBLib.CDS, WEBLib.ElectronCDS;
  {$ENDIF}

type
  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronPostgreSQLConnection = class(TElectronBaseConnection)
  private
    FSSL: TElectronNodeTLSSocketOptions;
  public
    constructor Create(aOwner : TComponent); override;
    destructor Destroy; override;
  published
    property User;
    property Host;
    property Password;
    property DatabaseName;
    property Port default 5432;
    property SSL: TElectronNodeTLSSocketOptions read FSSL write FSSL;
    property OnConnectionError;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronPostgreSQLClientDataSet = class(TElectronBaseClientDataSet)
  private
    FConnection: TElectronPostgreSQLConnection;
    procedure SetConnection(const Value: TElectronPostgreSQLConnection);
  protected
    procedure InternalClose; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalOpen; override;
    function IsCursorOpen: Boolean; override;
  published
    property Connection: TElectronPostgreSQLConnection read FConnection write SetConnection;
    property IndexName;
    property TableName;
  end;

implementation

{ TElectronPostgreSQLConnection }

constructor TElectronPostgreSQLConnection.Create(aOwner: TComponent);
begin
  inherited;
  Port := 5432;
  FSSL := TElectronNodeTLSSocketOptions.Create;
end;

destructor TElectronPostgreSQLConnection.Destroy;
begin
  FSSL.Free;
  inherited;
end;

{ TElectronPostgreSQLClientDataSet }

procedure TElectronPostgreSQLClientDataSet.InternalClose;
begin
  inherited;
end;

procedure TElectronPostgreSQLClientDataSet.InternalHandleException;
begin
  inherited;
end;

procedure TElectronPostgreSQLClientDataSet.InternalInitFieldDefs;
begin
  inherited;
end;

procedure TElectronPostgreSQLClientDataSet.InternalOpen;
begin
  inherited;
end;

function TElectronPostgreSQLClientDataSet.IsCursorOpen: Boolean;
begin
  Result := False;
end;

procedure TElectronPostgreSQLClientDataSet.SetConnection(
  const Value: TElectronPostgreSQLConnection);
begin
    if (FConnection = Value) then
    Exit;

  If Assigned(FCOnnection) then
    FConnection.RemoveFreeNotification(Self);

  FConnection := Value;

  If Assigned(FCOnnection) then
    FConnection.FreeNotification(Self);
end;

end.
