{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Forms;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, System.SysUtils, System.TypInfo, System.Rtti, Vcl.Controls,
  Vcl.Forms, Vcl.Graphics, JS, Web, WEBLib.Controls, WEBLib.Lang;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  SysUtils, TypInfo, Classes, Controls, Forms, Graphics, WEBLib.Controls,
  WEBLib.Lang, JS, Web, TMSWebCoreIDEOptions;
  {$ENDIF}

type
  {$IFNDEF LCLTMSWEB}
  TCloseAction = Vcl.Forms.TCloseAction;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TCloseAction = integer;
  {$ENDIF}

  TNavigationTarget = (ntBlank, ntPage);

  TFormBorderStyle = (fbNone, fbSingle, fbSizeable, fbDialog, fbDialogSizeable);
  TFormBorder = fbNone..fbSingle;
  TScreenOrientation = (soPortrait, soLandscape);

  TApplicationErrorType = (aeSilent, aeDialog, aeAlert, aeFooter);

  TModalResultProc = procedure(AValue: TModalResult);
  TFormCreatedProc = procedure(AForm: TObject);
  TBeforeUnloadEvent = procedure(Sender: TObject; var AMessage: string) of object;

  TOrientationChangeEvent = procedure(Sender: TObject; AOrientation: TScreenOrientation) of object;

  TCSSLibrary = (cssNone, cssBootstrap);

  {$IFNDEF LCLTMSWEB}
  TCustomWebForm = class(Vcl.Forms.TCustomForm)
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TCustomWebForm = class(TCustomForm)
  {$ENDIF}
  strict private
    FElementClassName: TElementClassName;
    FElementFont: TElementFont;
    FFormContainer: TElementID;
    FPopup: boolean;
    FPopupOpacity: single;
    FCaptionElement: TJSHTMLElement;
    FShadow: boolean;
    FBorder: TFormBorderStyle;
    FCSSLibrary: TCSSLibrary;
    function get_AlignWithMargins: Boolean;
    function get_CustomHint: TCustomHint;
    function get_HelpContext: THelpContext;
    function get_HelpKeyword: string;
    function get_HelpType: THelpType;
    function get_HorzScrollBar: TControlScrollBar;
    function get_Margins: TMargins;
    function get_OnAlignInsertBefore: TAlignInsertBeforeEvent;
    function get_OnAlignPosition: TAlignPositionEvent;
    function get_ParentCustomHint: Boolean;
    function get_VertScrollBar: TControlScrollBar;
    procedure ReadBoolean(AReader: TReader);
    procedure ReadInteger(AReader: TReader);
  private
    FElementPosition: TElementPosition;
  protected
    procedure DefineProperties(AFiler: TFiler); override;
  public
    constructor Create(AOwner: TComponent); override;
    constructor CreateNew(const AFileName: string); reintroduce; overload; virtual;
    constructor CreateNew; reintroduce; overload; virtual;
    constructor CreateNew(AProc: TFormCreatedProc); reintroduce; overload; virtual;
    constructor CreateNew(AElementID: string; AProc: TFormCreatedProc); reintroduce; overload; virtual;
    procedure SetEvent(const AInstance: TPersistent; const ALookupRoot: TComponent; const APropName, AMethodName: string);

    function ShowModal(AProc: TModalResultProc): TModalResult; reintroduce; overload; virtual;

    property AlignWithMargins: Boolean read get_AlignWithMargins;
    property Border: TFormBorderStyle read FBorder write FBorder;
    property CaptionElement: TJSHTMLElement read FCaptionElement;
    property CSSLibrary: TCSSLibrary read FCSSLibrary write FCSSLibrary default cssNone;
    property CustomHint: TCustomHint read get_CustomHint;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property FormContainer: TElementID read FFormContainer write FFormContainer;
    property HelpContext: THelpContext read get_HelpContext;
    property HelpKeyword: string read get_HelpKeyword;
    property HelpType: THelpType read get_HelpType;
    property HorzScrollBar: TControlScrollBar read get_HorzScrollBar;
    property Margins: TMargins read get_Margins;
    property ParentCustomHint: Boolean read get_ParentCustomHint;
    property Popup: boolean read FPopup write FPopup;
    property PopupOpacity: single read FPopupOpacity write FPopupOpacity;
    property Shadow: boolean read FShadow write FShadow;
    property VertScrollBar: TControlScrollBar read get_VertScrollBar;
    property OnAlignInsertBefore: TAlignInsertBeforeEvent read get_OnAlignInsertBefore;
    property OnAlignPosition: TAlignPositionEvent read get_OnAlignPosition;
  end;

  TWebForm = class(TCustomWebForm)
  private
    FOnTouchMove: TTouchEvent;
    FOnTouchStart: TTouchEvent;
    FOnTouchEnd: TTouchEvent;
    FOnScroll: TNotifyEvent;
    FOnUnload: TNotifyEvent;
    FOnBeforeUnload: TBeforeUnloadEvent;
    FOnOrientationChange: TOrientationChangeEvent;
    function GetColor: TColor;
    procedure SetColor(const Value: TColor);
  public
    {$IFDEF LCLTMSWEB}
    function ChildClassAllowed(ChildClass: TClass): boolean; override;
    {$ENDIF}
    procedure PreventDefault;
    property Position;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Caption;
    property Color: TColor read GetColor write SetColor default clWhite;
    property CSSLibrary;
    property Cursor;
    property CustomHint;
    property ElementClassName;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Font;
    property FormStyle;
    property FormContainer;
    property Height stored True;
    property HelpContext;
    property HelpKeyword;
    property HelpType;
    property Hint;
    property HorzScrollBar;
    property Left default 0;
    property Menu;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentCustomHint;
    property ParentFont default true;
    property ShowHint;
    property TabOrder default 1;
    property TabStop;
    property Tag;
    property Top default 0;
    property VertScrollBar;
    property Visible;
    property Width stored True;
    property OnAlignInsertBefore;
    property OnAlignPosition;
    property OnBeforeUnload: TBeforeUnloadEvent read FOnBeforeUnload write FOnBeforeUnload;
    property OnClick;
    property OnClose;
    property OnCloseQuery;
    property OnCreate;
    property OnDblClick;
    property OnDeactivate;
    property OnDestroy;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnOrientationChange: TOrientationChangeEvent read FOnOrientationChange write FOnOrientationChange;
    property OnPaint;
    property OnResize;
    property OnScroll: TNotifyEvent read FOnScroll write FOnScroll;
    property OnShow;
    property OnTouchStart: TTouchEvent read FOnTouchStart write FOnTouchStart;
    property OnTouchMove: TTouchEvent read FOnTouchMove write FOnTouchMove;
    property OnTouchEnd: TTouchEvent read FOnTouchEnd write FOnTouchEnd;
    property OnUnload: TNotifyEvent read FOnUnload write FOnUnload;
  end;

  TElectronFormBeforeUnloadEvent = procedure(Sender: TObject; var PreventClose: Boolean) of object;

  TElectronForm = class(TWebForm)
  private
    FOnBeforeElectronUnload: TElectronFormBeforeUnloadEvent;
  published
    property OnBeforeUnload: TElectronFormBeforeUnloadEvent read FOnBeforeElectronUnload write FOnBeforeElectronUnload;
  end;

  TWebFormClass = class of TWebForm;

  TAppplicationError = record
    AMessage: string;
    AFile: string;
    ALineNumber: integer;
    AColNumber: integer;
    AStack: string;
    AError: TJSObject;
  end;

  TOnlineStatus = (osOnline, osOffline);

  TAppplicationErrorEvent = procedure(Sender: TObject; AError: TAppplicationError; var Handled: boolean) of object;

  TApplicationHashChangeEvent = procedure(Sender: TObject; AHash: string; var Handled: boolean) of object;

  TApplicationOnlineChangeEvent = procedure(Sender: TObject; AStatus: TOnlineStatus) of object;

  TApplicationCallBackEvent = procedure(Sender: TObject; AQuery: string) of object;

  TApplicationColorScheme = (csNoScheme, csLight, csDark);

  TApplicationHelper = class helper for TApplication
  private
    function GetLanguage: TUILanguage;
    function GetOnImageCacheReady: TNotifyEvent;
    procedure SetOnImageCacheReady(const Value: TNotifyEvent);
    function GetOnFontCacheReady: TNotifyEvent;
    procedure SetOnFontCacheReady(const Value: TNotifyEvent);
    function GetOnError: TAppplicationErrorEvent;
    procedure SetOnError(const Value: TAppplicationErrorEvent);
    function GetAutoFormRoute: boolean;
    function GetOnHashChange: TApplicationHashChangeEvent;
    procedure SetAutoFormRoute(const Value: boolean);
    procedure SetOnHashChange(const Value: TApplicationHashChangeEvent);
    function GetErrorType: TApplicationErrorType;
    procedure SetErrorType(const Value: TApplicationErrorType);
    function GetOnOnlineChange: TApplicationOnlineChangeEvent;
    procedure SetOnOnlineChange(const Value: TApplicationOnlineChangeEvent);
    procedure SetThemeButtonClassName(const Value: string);
    procedure SetThemeColor(const Value: TColor);
    procedure SetThemeTextColor(const Value: TColor);
    function GetThemeButtonClassName: string;
    function GetThemeColor: TColor;
    function GetThemeTextColor: TColor;
    function GetIsOnline: boolean;
    function GetOnOAuthCallBack: TApplicationCallBackEvent;
    function GetOnOAuthToken: TApplicationCallBackEvent;
    procedure SetOnOAuthCallBack(const Value: TApplicationCallBackEvent);
    procedure SetOnOAuthToken(const Value: TApplicationCallBackEvent);
    function GetActiveForm: TForm;
    function GetAppContainer: TElementID;
    procedure SetAppContainer(const Value: TElementID);
    function GetColorScheme: TApplicationColorScheme;
    function GetThemed: Boolean;
    procedure SetThemed(const AValue: Boolean);
  public
    procedure CreateForm(AInstanceClass: TWebFormClass; AElementID: string; var AReference); overload;
    procedure CreateForm(AInstanceClass: TWebFormClass; AElementID: string; var AReference; AProc: TFormCreatedProc); overload;
    procedure Navigate(const AURL: string; ATarget: TNavigationTarget = ntBlank);
    procedure Download(const AURL: string);
    procedure DownloadTextFile(const AText: string; AFileName: string = '');
    procedure DownloadBinaryFile(const AData: TBytes; const AFileName: string = ''; const ANewTab: Boolean = False); overload;
    procedure DownloadBinaryFile(const AData: TJSUint8Array; const AFileName: string = ''; const ANewTab: Boolean = False); overload;
    procedure DownloadPDFFile(const AData: TBytes; const AFileName: string = ''; const ANewTab: Boolean = False); overload;
    procedure DownloadPDFFile(const AData: TJSUint8Array; const AFileName: string = ''; const ANewTab: Boolean = False); overload;
    procedure InitFormatSettings(const BrowserLocale: string);
    procedure ChangeCSS(id: string; href:string);
    procedure InsertCSS(id, href: string);
    function NeedsFormRouting: Boolean;
    procedure RemoveCSS(id: string);
    procedure RunScript(Source: string);
    procedure RouteForm(AParameter: string);
    procedure SetLanguage(const Value: TUILanguage);
    function ObjectURL(AFile: TJSHTMLFile): string;
    property AppContainer: TElementID read GetAppContainer write SetAppContainer;
    property ActiveForm: TForm read GetActiveForm;
    property AutoFormRoute: boolean read GetAutoFormRoute write SetAutoFormRoute;
    property ColorScheme: TApplicationColorScheme read GetColorScheme;
    property ErrorType: TApplicationErrorType read GetErrorType write SetErrorType;
    property IsOnline: boolean read GetIsOnline;
    property Language: TUILanguage read GetLanguage write SetLanguage;
    property ThemeColor: TColor read GetThemeColor write SetThemeColor;
    property ThemeTextColor: TColor read GetThemeTextColor write SetThemeTextColor;
    property ThemeButtonClassName: string read GetThemeButtonClassName write SetThemeButtonClassName;
    property Themed: Boolean read GetThemed write SetThemed;
    property OnImageCacheReady: TNotifyEvent read GetOnImageCacheReady write SetOnImageCacheReady;
    property OnFontCacheReady: TNotifyEvent read GetOnFontCacheReady write SetOnFontCacheReady;
    property OnError: TAppplicationErrorEvent read GetOnError write SetOnError;
    property OnHashChange: TApplicationHashChangeEvent read GetOnHashChange write SetOnHashChange;
    property OnOnlineChange: TApplicationOnlineChangeEvent read GetOnOnlineChange write SetOnOnlineChange;
    property OnOAuthCallBack: TApplicationCallBackEvent read GetOnOAuthCallBack write SetOnOAuthCallBack;
    property OnOAuthToken: TApplicationCallBackEvent read GetOnOAuthToken write SetOnOAuthToken;
  end;


{$IFDEF LCLTMSWEB}
  TValue = variant;
{$ENDIF}

function GetOwnerCSSLibrary(AOwner: TComponent): TCSSLibrary;

procedure Log(v: TValue); overload;
procedure Log(arr: array of TValue); overload;

function Application: TApplication;

implementation

function GetOwnerCSSLibrary(AOwner: TComponent): TCSSLibrary;
var
  frm: TCustomForm;
begin
  Result := cssNone;
  frm := nil;

  if AOwner is TWebForm then
  begin
    frm := AOwner as TWebForm;
  end
  else
    if (AOwner is TControl) then
      frm := GetParentForm(AOwner as TControl);

  if Assigned(frm) then
    if frm is TWebForm then
    begin
      Result := (frm as TWebForm).CSSLibrary;
    end;
end;

function Application: TApplication;
begin
  {$IFNDEF LCLTMSWEB}
  Result := Vcl.Forms.Application;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := Application;
  {$ENDIF}
end;

procedure Log(v: TValue);
begin
  //
end;

procedure Log(arr: array of TValue);
begin
  //
end;

{ TCustomWebForm }


constructor TCustomWebForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  BorderStyle := bsNone;
  Color := clWhite;
  FPopup := false;
  FPopupOpacity := 1;
  Font.Name := WEBDEFAULTFONT;
  ElementFont := efProperty;
  ParentFont := True;
  TabOrder := 0;
end;

constructor TCustomWebForm.CreateNew(const AFileName: string);
begin
  Create(nil);
end;

constructor TCustomWebForm.CreateNew;
begin
  //
end;

constructor TCustomWebForm.CreateNew(AProc: TFormCreatedProc);
begin
  //
end;

constructor TCustomWebForm.CreateNew(AElementID: string; AProc: TFormCreatedProc);
begin
  //
end;


procedure TCustomWebForm.DefineProperties(AFiler: TFiler);
const
  cClientHeight = 'ClientHeight';
  cClientWidth = 'ClientWidth';
  cPixelsPerInch = 'PixelsPerInch';
  cOldCreateOrder = 'OldCreateOrder';
  cTextHeight = 'TextHeight';
  cIgnoreFontProperty = 'IgnoreFontProperty';
begin
  AFiler.DefineProperty(cClientHeight, ReadInteger, nil, False);
  AFiler.DefineProperty(cClientWidth, ReadInteger, nil, False);
  AFiler.DefineProperty(cOldCreateOrder, ReadBoolean, nil, False);
  AFiler.DefineProperty(cPixelsPerInch, ReadInteger, nil, False);
  AFiler.DefineProperty(cTextHeight, ReadInteger, nil, False);
  AFiler.DefineProperty(cIgnoreFontProperty, ReadBoolean, nil, False);
end;

function TCustomWebForm.get_AlignWithMargins: Boolean;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited AlignWithMargins;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := false;
  {$ENDIF}
end;

function TCustomWebForm.get_CustomHint: TCustomHint;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited CustomHint;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := inherited Hint;
  {$ENDIF}
end;

function TCustomWebForm.get_HelpContext: THelpContext;
begin
  Result := inherited HelpContext;
end;

function TCustomWebForm.get_HelpKeyword: string;
begin
  Result := inherited HelpKeyword;
end;

function TCustomWebForm.get_HelpType: THelpType;
begin
  Result := inherited HelpType;
end;

function TCustomWebForm.get_HorzScrollBar: TControlScrollBar;
begin
  Result := inherited HorzScrollBar;
end;

function TCustomWebForm.get_Margins: TMargins;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited Margins;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := -1;
  {$ENDIF}
end;

function TCustomWebForm.get_OnAlignInsertBefore: TAlignInsertBeforeEvent;
begin
  Result := inherited OnAlignInsertBefore;
end;

function TCustomWebForm.get_OnAlignPosition: TAlignPositionEvent;
begin
  Result := inherited OnAlignPosition;
end;

function TCustomWebForm.get_ParentCustomHint: Boolean;
begin
  {$IFNDEF LCLTMSWEB}
  Result := inherited ParentCustomHint;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Result := false;
  {$ENDIF}
end;

function TCustomWebForm.get_VertScrollBar: TControlScrollBar;
begin
  Result := inherited VertScrollBar;
end;

procedure TCustomWebForm.ReadBoolean(AReader: TReader);
begin
  AReader.ReadBoolean;
end;

procedure TCustomWebForm.ReadInteger(AReader: TReader);
begin
  AReader.ReadInteger;
end;

procedure TCustomWebForm.SetEvent(const AInstance: TPersistent; const ALookupRoot: TComponent; const APropName, AMethodName: string);
var
  lMethod: TMethod;
begin
  lMethod.Code := ALookupRoot.MethodAddress(AMethodName);
  if lMethod.Code = nil then
    raise Exception.Create(ALookupRoot.ClassName +' has no published method "'+ AMethodName + '"');
   lMethod.Data := ALookupRoot;
  SetMethodProp(AInstance, APropName, lMethod);
end;

function TCustomWebForm.ShowModal(AProc: TModalResultProc): TModalResult;
begin
  Result := ShowModal;
  if Assigned(AProc) then
    AProc(ModalResult);
end;

{ TWebForm }

function TWebForm.GetColor: TColor;
begin
  Result := inherited Color;
end;

{$IFDEF LCLTMSWEB}
function TWebForm.ChildClassAllowed(ChildClass: TClass): boolean;
begin
  if ChildClass.InheritsFrom(TComponent) then
    Result := WebCoreOptions.IsComponentAllowed(TComponentClass(ChildClass))
  else
    Result := true;
end;
{$ENDIF}

procedure TWebForm.PreventDefault;
begin

end;

procedure TWebForm.SetColor(const Value: TColor);
begin
  inherited Color := Value;
end;

{ TApplicationHelper }

procedure TApplicationHelper.CreateForm(AInstanceClass: TWebFormClass;
  AElementID: string; var AReference);
begin
 //
end;

procedure TApplicationHelper.ChangeCSS(id, href: string);
begin
  //
end;

procedure TApplicationHelper.CreateForm(AInstanceClass: TWebFormClass;
  AElementID: string; var AReference; AProc: TFormCreatedProc);
begin
 //
end;


procedure TApplicationHelper.Download(const AURL: string);
begin
 //
end;

procedure TApplicationHelper.DownloadTextFile(const AText: string; AFileName: string = '');
begin
  //
end;

procedure TApplicationHelper.DownloadBinaryFile(const AData: TBytes; const AFileName: string; const ANewTab: Boolean);
begin
  //
end;

procedure TApplicationHelper.DownloadBinaryFile(const AData: TJSUint8Array; const AFileName: string; const ANewTab: Boolean);
begin
  //
end;

procedure TApplicationHelper.DownloadPDFFile(const AData: TBytes; const AFileName: string; const ANewTab: Boolean);
begin
  //
end;

procedure TApplicationHelper.DownloadPDFFile(const AData: TJSUint8Array; const AFileName: string; const ANewTab: Boolean);
begin
  //
end;

function TApplicationHelper.GetActiveForm: TForm;
begin
  Result := nil;
end;

function TApplicationHelper.GetAppContainer: TElementID;
begin
  Result := 'body';
end;

function TApplicationHelper.GetAutoFormRoute: boolean;
begin
  Result := false;
end;

function TApplicationHelper.GetColorScheme: TApplicationColorScheme;
begin
  Result := csNoScheme;
end;

function TApplicationHelper.GetErrorType: TApplicationErrorType;
begin
  Result := aeSilent;
end;

function TApplicationHelper.GetIsOnline: boolean;
begin
  Result := true;
end;

function TApplicationHelper.GetLanguage: TUILanguage;
begin
  Result := lNone;
end;

function TApplicationHelper.GetOnError: TAppplicationErrorEvent;
begin
  //
end;

function TApplicationHelper.GetOnFontCacheReady: TNotifyEvent;
begin
  //
end;

function TApplicationHelper.GetOnHashChange: TApplicationHashChangeEvent;
begin
  //
end;

function TApplicationHelper.GetOnImageCacheReady: TNotifyEvent;
begin
  //
end;

function TApplicationHelper.GetOnOAuthCallBack: TApplicationCallBackEvent;
begin
  //
end;

function TApplicationHelper.GetOnOAuthToken: TApplicationCallBackEvent;
begin
  //
end;

function TApplicationHelper.GetOnOnlineChange: TApplicationOnlineChangeEvent;
begin
  //
end;

function TApplicationHelper.GetThemeButtonClassName: string;
begin
  //
end;

function TApplicationHelper.GetThemeColor: TColor;
begin
  Result := clBlue;
end;

function TApplicationHelper.GetThemeTextColor: TColor;
begin
  Result := clWhite;
end;

function TApplicationHelper.GetThemed: Boolean;
begin
  Result := False;
end;

procedure TApplicationHelper.InitFormatSettings(const BrowserLocale: string);
begin
  //
end;

procedure TApplicationHelper.InsertCSS(id, href: string);
begin
  //
end;

procedure TApplicationHelper.Navigate(const AURL: string;
  ATarget: TNavigationTarget);
begin
  //
end;

function TApplicationHelper.NeedsFormRouting: Boolean;
begin
  Result := False;
end;

function TApplicationHelper.ObjectURL(AFile: TJSHTMLFile): string;
begin
  //
end;

procedure TApplicationHelper.RemoveCSS(id: string);
begin
  //
end;

procedure TApplicationHelper.RouteForm(AParameter: string);
begin
  //
end;

procedure TApplicationHelper.RunScript(Source: string);
begin
  //
end;

procedure TApplicationHelper.SetAppContainer(const Value: TElementID);
begin
  //
end;

procedure TApplicationHelper.SetAutoFormRoute(const Value: boolean);
begin
  //
end;

procedure TApplicationHelper.SetErrorType(const Value: TApplicationErrorType);
begin
///
end;

procedure TApplicationHelper.SetLanguage(const Value: TUILanguage);
begin
  //
end;

procedure TApplicationHelper.SetOnError(const Value: TAppplicationErrorEvent);
begin
  //
end;

procedure TApplicationHelper.SetOnFontCacheReady(const Value: TNotifyEvent);
begin
  //
end;

procedure TApplicationHelper.SetOnHashChange(
  const Value: TApplicationHashChangeEvent);
begin
  //
end;

procedure TApplicationHelper.SetOnImageCacheReady(const Value: TNotifyEvent);
begin
  //
end;

procedure TApplicationHelper.SetOnOAuthCallBack(
  const Value: TApplicationCallBackEvent);
begin
  //
end;

procedure TApplicationHelper.SetOnOAuthToken(
  const Value: TApplicationCallBackEvent);
begin
  //
end;

procedure TApplicationHelper.SetOnOnlineChange(
  const Value: TApplicationOnlineChangeEvent);
begin
  //
end;

procedure TApplicationHelper.SetThemeButtonClassName(const Value: string);
begin
  //
end;

procedure TApplicationHelper.SetThemeColor(const Value: TColor);
begin
  //
end;

procedure TApplicationHelper.SetThemeTextColor(const Value: TColor);
begin
  //
end;

procedure TApplicationHelper.SetThemed(const AValue: Boolean);
begin
  //
end;

end.
