{******************************************************************************}
{ Mini HTML rendering engine                                                   }
{ for Delphi & C++Builder                                                      }
{                                                                              }
{ written by TMS Software                                                      }
{            copyright  1999 - 2018                                           }
{            Email : info@tmssoftware.com                                      }
{            Website : http://www.tmssoftware.com/                             }
{                                                                              }
{ The source code is given as is. The author is not responsible                }
{ for any possible damage done due to the use of this code.                    }
{ The component can be freely used in any application. The complete            }
{ source code remains property of the author and may not be distributed,       }
{ published, given or sold in any form as such. No parts of the source         }
{ code can be included in any other component or application without           }
{ written authorization of the author.                                         }
{******************************************************************************}

unit WEBLib.HTML;

interface

uses
  {$IFNDEF LCLTMSWEB}
  WinApi.Windows, Vcl.Graphics, System.Classes, Vcl.Controls, Vcl.ComCtrls, Vcl.Dialogs, Vcl.ImgList,
  WinApi.Messages, System.Types, System.UITypes, ShellAPI, CommCtrl
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Graphics, Classes, Types, LCLType, LCLIntf, Controls, ComCtrls, Dialogs,
  ImgList, Messages
  {$IFDEF Windows}
  ,ShellAPI, CommCtrl
  {$ENDIF}
  {$ENDIF}
  ;

function HTMLDrawEx(Canvas:TCanvas; s:string; fr:TRect;
                    FImages: TCustomImageList;
                    XPos,YPos,FocusLink,HoverLink,ShadowOffset: Integer;
                    CheckHotSpot,CheckHeight,Print,Selected,Blink,HoverStyle,WordWrap,Down: Boolean;
                    DownID: string; ResFactor:Double;
                    URLColor,HoverColor,HoverFontColor,ShadowColor:TColor;
                    var AnchorVal,StripVal,FocusAnchor,AnchorHint: string;
                    var XSize,YSize,HyperLinks,MouseLink: Integer;
                    var HoverRect,ControlRect:TRect;var CID,CV,CT: string
                    ): Boolean;

function HTMLStrip(s:string):string;

implementation

uses
  SysUtils;

const
  NumSpecialChar = 56;
  WL_ATTR_DELIM = '"';
  WL_KEEP_LINEBREAKS = true;

type
  TColorentry = record
    sc:  pchar;
    tc:  tcolor;
  end;

const // color names with the cl removed
  // could easily be extended
  colorTable: array[0..39] of TColorEntry = ( (sc:'3ddkshadow';tc:clgraytext),
  (sc:'3dlight';tc:cl3dlight),
  (sc:'activeborder';tc:clActiveborder),
  (sc:'activecaption';tc:clActivecaption),
  (sc:'appworkspace';tc:clAppworkspace),
  (sc:'aqua';tc:claqua),
  (sc:'background';tc:clBackground),
  (sc:'black';tc:clBlack),
  (sc:'blue';tc:clBlue),
  (sc:'btnface';tc:clBtnface),
  (sc:'btnhighlight';tc:clBtnhighlight),
  (sc:'btnshadow';tc:clBtnshadow),
  (sc:'btntext';tc:clBtntext),
  (sc:'captiontext';tc:clCaptiontext),
  (sc:'fuchsia';tc:clFuchsia),
  (sc:'gray';tc:clGray),
  (sc:'graytext';tc:clGraytext),
  (sc:'green';tc:clGreen),
  (sc:'highlight';tc:clHighlight),
  (sc:'highlighttext';tc:clHighlighttext),
  (sc:'inactiveborder';tc:clInactiveborder),
  (sc:'inactivecaption';tc:clInactivecaption),
  (sc:'inactivecaptiontext';tc:clInactivecaptiontext),
  (sc:'infobk';tc:clInfobk),
  (sc:'infotext';tc:clInfotext),
  (sc:'lime';tc:clLime),
  (sc:'maroon';tc:clMaroon),
  (sc:'menu';tc:clMenu),
  (sc:'menutext';tc:clMenutext),
  (sc:'navy';tc:clNavy),
  (sc:'olive';tc:clOlive),
  (sc:'purple';tc:clPurple),
  (sc:'red';tc:clRed),
  (sc:'silver';tc:clSilver),
  (sc:'teal';tc:clTeal),
  (sc:'white';tc:clWhite),
  (sc:'window';tc:clWindow),
  (sc:'windowframe';tc:clWindowframe),
  (sc:'windowtext';tc:clWindowtext),
  (sc:'yellow';tc:clYellow));


var
  HTMLHighlightColor: TColor = clHighlight;
  HTMLHighlightTextColor: TColor = clHighlightText;

  HTMLEncodedChar : array[1..56] of ansistring = ('','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','',
                                             '','','','');

  HTMLSpecialChar : array[1..56] of ansistring = ('&eacute;','&egrave;','&euml;','&ecirc;',
                                             '&oacute;','&ograve;','&ouml;','&ocirc;',
                                             '&iacute;','&igrave;','&iuml;','&icirc;',
                                             '&uacute;','&ugrave;','&uuml;','&ucirc;',
                                             '&aacute;','&agrave;','&auml;','&acirc;',
                                             '&Eacute;','&Egrave;','&Euml;','&Ecirc;',
                                             '&Oacute;','&Ograve;','&Ouml;','&Ocirc;',
                                             '&Iacute;','&Igrave;','&Iuml;','&Icirc;',
                                             '&Uacute;','&Ugrave;','&Uuml;','&Ucirc;',
                                             '&Aacute;','&Agrave;','&Auml;','&Acirc;',
                                             '&ccedil;','&Ccedil;','&oslash;','&Oslash;',
                                             '&aring;','&Aring;','&copy;','&reg;',
                                             '&euro;','&laquo;','&raquo;','&atilde;','&Atilde;','&otilde;','&Otilde','&szlig;');

procedure SetHighLightColors(AColor, ATextColor: TColor);
begin
  HTMLHighlightColor := AColor;
  HTMLHighlightTextColor := ATextColor;
end;

function DirExists(const Name: string): Boolean;
{$IFDEF LCLTMSWEB}
begin
  Result:=DirectoryExists(Name);
end;
{$ELSE}
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Name));

  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;
{$ENDIF}

function SysImage(Canvas: TCanvas;x,y:Integer;APath:string;large,draw,print:boolean;resfactor:double):TPoint;
{$IFDEF WINDOWS}
var
  SFI: TSHFileInfo;
  i,Err: Integer;
  imglsthandle: THandle;
  rx,ry: Integer;
begin
  Val(APath,i,Err);

  if (APath <> '') and (Err <> 0) then
  begin
    if FileExists(APath) or DirExists(APath) then
    // If the file or directory exists, just let Windows figure out it's attrs.
      SHGetFileInfo(PChar(APath), 0, SFI, SizeOf(TSHFileInfo),
                    SHGFI_SYSICONINDEX {or OPEN_FLAG[Open] or SELECTED_FLAG[Selected]})
    else
    // File doesn't exist, so Windows doesn't know what to do with it.  We have
    // to tell it by passing the attributes we want, and specifying the
    // SHGFI_USEFILEATTRIBUTES flag so that the function knows to use them.
      SHGetFileInfo(PChar(APath), 0, SFI, SizeOf(TSHFileInfo),
                    SHGFI_SYSICONINDEX or SHGFI_USEFILEATTRIBUTES {or OPEN_FLAG[Open] or SELECTED_FLAG[Selected]});
    i := SFI.iIcon;
  end;

  if Large then
    imglsthandle := SHGetFileInfo('', 0, SFI, SizeOf(SFI),
                            SHGFI_SYSICONINDEX or SHGFI_LARGEICON)
  else
    imglsthandle := SHGetFileInfo('', 0, SFI, SizeOf(SFI),
                            SHGFI_SYSICONINDEX or SHGFI_SMALLICON);


  ImageList_GetIconSize(imglsthandle,rx,ry);
  Result := Point(rx,ry);

  if Draw and not Print then
    ImageList_Draw(imglsthandle,i,Canvas.handle,x,y, ILD_TRANSPARENT);
end;
{$ELSE}
begin
  if Canvas = nil then ;
  if x + y = 0 then ;
  if APath = '' then ;
  if large and draw and print then ;
  if resfactor = 0.0 then ;
  Result := Point(0,0);
end;
{$ENDIF}

function Str2hexbyte(src: string; ndx: integer): integer; var
  n:    integer;
  i:    integer;
begin
  Result := 0;
  if not (length(src) > ndx) then
    Exit; // or raise an exception src is not big enough to hold a hex byte
  for i := 0 to 1 do
  begin
    n := ord(upcase(src[ndx+i])) - ord('0');
    if n > 9 then
      n := n - 7;
    if (n < 0) or (n > 15) then
    begin
      Result := 0;  // invalid hex digit
      Exit;         // might be better to raise an exception as alternate action
    end;            // so programmer would know the entry was wrong
    Result := Result shl 4 or n;
  end;
end { Str2HexByte };


function Text2Color(s:string):tcolor;
var
  r,g,b: Integer;
  top, bot, cur:  integer;
  key:  pchar;
begin
  if s[1] = '#' then
  begin
    r := Str2hexbyte(s,2);
    g := Str2hexbyte(s,4) shl 8;
    b := Str2Hexbyte(s,6) shl 16;
    Result := TColor(b + g + r);
    exit;
  end;
  Result := clBlack; // 40 colors
  if pos('cl',s) <> 1 then
    Exit;   // all colors start with cl

  // do a binary search for chosen color -- max 6 comparisons
  key := pchar(s)+2; // skip the cl in front of color name
  bot := 0;
  top := high(colortable);
  while bot <= top do
  begin
    cur := (bot + top) div 2;
    if strComp(colortable[cur].sc,key) < 0 then
      bot := cur + 1
    else
      top := cur - 1;
  end;
  // one more comparison to verify exact match
  if strcomp(colortable[bot].sc,key) = 0 then
    result := colortable[bot].tc;
end { TextToColor };


function HexVal(s:string): Integer;
var
  i,j: Integer;
begin
  if Length(s) < 2 then
  begin
    Result := 0;
    Exit;
  end;

  if s[1] >= 'A' then
    i := ord(s[1]) - ord('A') + 10
  else
    i := ord(s[1]) - ord('0');

  if s[2] >= 'A' then
    j := ord(s[2]) - ord('A') + 10
  else
    j := ord(s[2]) - ord('0');

  Result := i shl 4 + j;
end;

function Hex2Color(s:string): TColor;
var
  r,g,b: Integer;
begin
  r := Hexval(Copy(s,2,2));
  g := Hexval(Copy(s,4,2)) shl 8;
  b := Hexval(Copy(s,6,2)) shl 16;
  Result := TColor(b + g + r);
end;

function IPos(su,s:string):Integer;
begin
  Result := Pos(UpperCase(su),UpperCase(s));
end;

function IStrToInt(s:string):Integer;
var
  Err,Res: Integer;
begin
  Val(s,Res,Err);
  Result := Res;
end;

function UnicodeTagStrip(s: string): string;
var
  i,j,e: Integer;
  su,res: string;
  w: word;

begin
  res := '';
  repeat
    i := Pos('&#',s);
    if (i > 0) then
    begin
      res := res + Copy(s,1,i - 1);
      Delete(s,1,i - 1);
      j := Pos(';',s);
      if j > 0 then
      begin
        su := Copy(s,3,j - 3);
        val(su, w, e);

        if (length(su)> 0) and (length(su) <= 4) and (e = 0) then
        begin
          res := res + chr(w);
          Delete(s,1,j);
        end;
      end
      else
      begin
        res := res + '&#';
        Delete(s,1,2);
      end;
    end
    else
      res := res + s;
  until i <= 0;

  Result := res;
end;

function DBTagStrip(s:string):string;
var
  i,j: Integer;
begin
  repeat
    i := Pos('<#',s);
    if i > 0 then
      begin
        Result := Copy(s,1,i - 1);
        Delete(s,1,i);
        j := Pos('>',s);
        if j > 0 then
          Delete(s,j,1);
        Result := Result + s;
        s := Result;
      end
    else
      Result := s;
  until (i <= 0);
end;

function CRLFStrip(s: string; break:boolean):string;
var
  i,ls: Integer;
  lc,nc: char;

begin
  Result := '';
  ls := Length(s);

  for i := 1 to ls do
  begin
    if i > 1 then
      lc := s[i - 1]
    else
      lc := #0;

    if i + 1 < ls then
      nc := s[i + 1]
    else
      nc := #0;

    if not ( (s[i] = #13) or (s[i] = #10) ) then
      Result := Result + s[i]
    else
      if (s[i] = #13) then
        if break then
          Result := Result + '<BR>'
        else
        begin
          if (lc <> '>') or (nc <> '<') then
            Result := Result + ' ';
        end;
  end;
end;

function VarPos(su,s:string;var Res:Integer):Integer;
begin
  Res := Pos(su,s);
  Result := Res;
end;

function VarIPos(su,s:string;var Res:Integer):Integer;
begin
  Res := Pos(su,Uppercase(s));
  Result := Res;
end;


function TagReplaceString(const Srch,Repl:string;var Dest:string):Boolean;
var
  i: Integer;
begin
  i := IPos(srch,dest);
  if i > 0 then
  begin
    Result := True;
    Delete(Dest,i,Length(Srch));
    Dest := Copy(Dest,1,i-1) + Repl + Copy(Dest,i,Length(Dest));
  end
  else
    Result := False;
end;

function FirstChar(sub:string; s: string): integer;
var
  i,j: integer;
begin
  Result := -1;

  for i := 1 to Length(s) do
  begin
    for j := 1 to Length(sub) do
    begin
      if sub[j] = s[i] then
      begin
        Result := i;
        Exit;
      end;
    end;
  end;
end;


function UnFixMarkup(su:string;SpecialChars: boolean = true):string;
var
  i: integer;
begin
  while Pos('&quot;',su) > 0 do
  begin
    TagReplacestring('&quot;','"',su);
  end;

  if SpecialChars then
  begin
    for i := 1 to NumSpecialChar do
    begin
      while Pos(string(HTMLSpecialChar[i]),su) > 0 do
      begin
        TagReplacestring(string(HTMLSpecialChar[i]),string(HTMLEncodedChar[i]),su);
      end;
    end;
  end;

  Result := su;
end;

function FixMarkup(su: string; SpecialChars: boolean = true): string;
var
  i: integer;
begin
  while Pos('&',su) > 0 do
  begin
    TagReplacestring('&','*amp;',su);
  end;

  while Pos('*amp;',su) > 0 do
  begin
    TagReplacestring('*amp;','&amp;',su);
  end;

  while Pos('"',su) > 0 do
  begin
    TagReplacestring('"','&quot;',su);
  end;

  while Pos('<',su) > 0 do
  begin
    TagReplacestring('<','&lt;',su);
  end;

  while Pos('>',su) > 0 do
  begin
    TagReplacestring('>','&gt;',su);
  end;

  if SpecialChars then
  begin
    for i := 1 to NumSpecialChar do
    begin
      while Pos(string(HTMLEncodedChar[i]),su) > 0 do
      begin
        TagReplacestring(string(HTMLEncodedChar[i]),string(HTMLSpecialChar[i]),su);
      end;
    end;
  end;

  Result := su;
end;

function FixNonBreaking(su: string): string;
begin
  while Pos(' ',su) > 0 do
  begin
    TagReplacestring(' ','&nbsp;',su);
  end;
  Result := su;
end;

procedure ParseControl(Tag: string; var ControlType,ControlID,ControlValue,ControlWidth,ControlHeight,ControlProp,ControlLen:string);
var
  Prop: string;
  vp: integer;
begin
  ControlType := '';
  ControlWidth := '';
  ControlHeight := '';
  ControlValue := '';
  ControlID := '';
  ControlProp := '';
  ControlLen := '';

  if VarIPos('TYPE=',Tag,vp) > 0 then
  begin
    Prop := Copy(Tag,vp + 1,Length(Tag));
    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
    ControlType := Uppercase(Prop);
  end;

  if VarIPos('WIDTH=',Tag,vp) > 0 then
  begin
    Prop := Copy(Tag,vp + 1,Length(Tag));
    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
    ControlWidth := Prop;
  end;

  if VarIPos('HEIGHT=',Tag,vp) > 0 then
  begin
    Prop := Copy(Tag,vp + 1,Length(Tag));
    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
    ControlHeight := Prop;
  end;

  if VarIPos('ID=',Tag,vp) > 0 then
  begin
    Prop := Copy(Tag,vp + 1,Length(Tag));
    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
    ControlID := Prop;
  end;

  if VarIPos('VALUE=',Tag,vp) > 0 then
  begin
    Prop := Copy(Tag,vp + 1,Length(Tag));
    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
    ControlValue := UnFixMarkup(Prop);
  end;

  if VarIPos('PROP=',Tag,vp) > 0 then
  begin
    Prop := Copy(Tag,vp + 1,Length(Tag));
    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
    ControlProp := UnFixMarkup(Prop);
  end;

  if VarIPos('MAXLEN=',Tag,vp) > 0 then
  begin
    Prop := Copy(Tag,vp + 1,Length(Tag));
    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
    ControlLen := Prop;
  end;
end;

function ConvertToPassword(s:string):string;
var
  i: Integer;
begin
  Result := '';
  for i := 1 to Length(s) do
    Result := Result + '*';
end;

{$IFDEF LCLTMSWEB}
const
  DT_EXTERNALLEADING = 0;

function DrawTextEx(DC: HDC; Str: PChar; Count: Integer; var Rect: TRect; Flags: Cardinal; p: pointer): Integer;
begin
  Result := DrawText(DC,Str,Count,Rect,Flags);
end;
{$ENDIF}

{$WARNINGS OFF}
function HTMLDrawEx(Canvas:TCanvas; s:string; fr:TRect;
                    FImages: TCustomImageList;
                    XPos,YPos,FocusLink,HoverLink,ShadowOffset: Integer;
                    CheckHotSpot,CheckHeight,Print,Selected,Blink,HoverStyle,WordWrap,Down: Boolean;
                    DownID: string; ResFactor:Double;
                    URLColor,HoverColor,HoverFontColor,ShadowColor:TColor;
                    var AnchorVal,StripVal,FocusAnchor,AnchorHint: string;
                    var XSize,YSize,HyperLinks,MouseLink: Integer;
                    var HoverRect,ControlRect:TRect;var CID,CV,CT: string
                    ): Boolean;

type
  TFloatStyle = (fsNone, fsLeft, fsRight);

var
  su: string;
  r,dr,hr,rr: TRect;
  htmlwidth,htmlheight,txtheight,ch,fc: Integer;
  Align: TAlignment;
  PIndent: Integer;
  OldFont: TFont;
  CalcFont: TFont;
  DrawFont: TFont;
  OldCalcFont: TFont;
  OldDrawFont: TFont;
  Hotspot, ImageHotspot: Boolean;
  Anchor,OldAnchor,MouseInAnchor,Error: Boolean;
  bgcolor,paracolor,hvrcolor,pencolor,blnkcolor,hifcol,hibcol: TColor;
  LastAnchor,OldAnchorVal,LastAnchorHint, OldAnchorHint: string;
  IMGSize: TPoint;
  isSup,isSub,isPara,isShad,isHi: Boolean;
  subh,suph,imgali,hlcount,licount: Integer;
  holdfont: THandle;
  ListIndex,OListIndex: Integer;
  Invisible: Boolean;
  FoundTag: Boolean;
  {new for editing}
  nnFit: Integer;
  nnSize: TSize;
  inspoint: Integer;
  AltImg,ImgIdx,OldImgIdx: Integer;
  DrawStyle: DWord;
  FHot: Boolean;
  UseWinXP: Boolean;
  OrgPenColor: TColor;
  OrgPenWidth: Integer;
  OLIST: boolean;
  OListIndexStr: string;
  floatrect: TRect;
  floatpt: TPoint;
  txtfloat,txtfloatbefore: TFloatStyle;
  imgfloat: TFloatStyle;
  scale: single;

  procedure StartRotated(Canvas: TCanvas;Angle: Integer);
  var
    LFont:TLogFont;
  begin
    GetObject(Canvas.Font.Handle,SizeOf(LFont),Addr(LFont));

    LFont.lfEscapement := Angle * 10;
    LFont.lfOrientation := Angle * 10;
    hOldFont := SelectObject(Canvas.Handle,CreateFontIndirect(LFont));
  end;

  procedure EndRotated(Canvas:TCanvas);
  begin
    DeleteObject(SelectObject(Canvas.Handle,hOldFont));
  end;

  function HTMLDrawLine(Canvas: TCanvas;var s:string;r: TRect;Calc:Boolean;
                        var w,h,subh,suph,imgali:Integer;var Align:TAlignment; var PIndent: Integer;
                        XPos,YPos:Integer;var Hotspot,ImageHotSpot:Boolean; var floatrect: TRect; var txtfloat: TFloatStyle): string;
  var
    su,Res,TagProp,Prop,Tagp,LineText:string;
    cr,ir,flr: TRect;
    linebreak,imgbreak,linkbreak: Boolean;
    th,sw,indent,err,imgw,imgh,spc: Integer;
    TagPos,SpacePos,o,l: Integer;
    bmp: TGraphic;
    NewColor: TColor;
    TagWidth,TagHeight,WordLen,WordLenEx,WordWidth: Integer;
    TagChar: Char;
    LengthFits: Boolean;
    ControlType,ControlWidth,ControlHeight,ControlID,ControlValue,ControlProp,ControlLen: string;
    txth: integer;

  begin
    Result := '';
    LineText := '';
    r.Bottom := r.Bottom - Subh;

    w := 0;
    sw := 0;

    LineBreak := False;
    ImgBreak := False;
    LinkBreak := False;
    HotSpot := False;
    ImageHotSpot := False;
    cr := r;
    res := '';

    if isPara and not Calc then
    begin
      Pencolor := Canvas.Pen.Color;
      Canvas.Pen.color := Canvas.Brush.Color;
      Canvas.Rectangle(fr.Left,r.Top,fr.Right,r.Top + h);
    end;

    while (Length(s) > 0) and not LineBreak and not ImgBreak do
    begin
      // get next word or till next HTML tag
      TagPos := Pos('<',s);

      if WordWrap then
        SpacePos := Pos(' ',s)
      else
        SpacePos := 0;

      if (Tagpos > 0) and ((SpacePos > TagPos) or (SpacePos = 0)) then
      begin
        su := Copy(s,1,TagPos - 1);
      end
      else
      begin
        if SpacePos > 0 then
          su := Copy(s,1,SpacePos)
        else
          su := s;
      end;

      {$IFDEF TMSDEBUG}
      DbgMsg(su+ '.');
      {$ENDIF}

      WordLen := Length(su);

      while Pos('&nbsp;',su) > 0 do
      begin
        TagReplacestring('&nbsp;',' ',su);
      end;

      while Pos('&lt;',su) > 0 do
      begin
        TagReplacestring('&lt;','<',su);
      end;

      while Pos('&gt;',su) > 0 do
      begin
        TagReplacestring('&gt;','>',su);
      end;

      for spc := 1 to NumSpecialChar do
      begin
        while Pos(string(HTMLSpecialChar[spc]),su) > 0 do
        begin
          TagReplacestring(string(HTMLSpecialChar[spc]),string(HTMLEncodedChar[spc]),su);
        end;
      end;

      WordLenEx := Length(su);
      WordWidth := 0;

      if WordLen > 0 then
      begin
        th := Canvas.TextHeight(su);

        if isSub and (subh < (th shr 2)) then subh := th shr 2;
        if isSup and (suph < (th shr 2)) then suph := th shr 2;

        if th > h then
          h := th;

        StripVal := StripVal + su;

        if Invisible then
          Delete(s,1,WordLen);

        if not Invisible then
        begin
          // draw mode
          if not Calc then
          begin
            if isSup then
              cr.Bottom := cr.Bottom - suph;
            if isSub then
              cr.Bottom := cr.Bottom + subh;

            cr.Bottom := cr.Bottom - imgali;

            if isShad then
            begin
              OffsetRect(cr,ShadowOffset,ShadowOffset);
              NewColor := Canvas.Font.Color;
              Canvas.Font.Color := ShadowColor;
              DrawTextEx(Canvas.Handle,PChar(su),WordLenEx,cr,DrawStyle,nil);
              Offsetrect(cr,-ShadowOffset,-ShadowOffset);
              Canvas.Font.Color := NewColor;
            end;

            err := cr.Right;

            DrawTextEx(Canvas.Handle,PChar(su),WordLenEx,cr,DrawStyle,nil);
            DrawTextEx(Canvas.Handle,PChar(su),WordLenEx,cr,DrawStyle or DT_CALCRECT,nil);

            if Anchor and (Hyperlinks - 1 = FocusLink) then
              FocusAnchor := LastAnchor;

            {$IFDEF TMSDEBUG}
            if Anchor then
              DbgMsg('drawrect for '+anchorval+' = ['+inttostr(cr.Left)+':'+inttostr(cr.Top)+'] ['+inttostr(cr.right)+':'+inttostr(cr.bottom)+'] @ ['+inttostr(xpos)+':'+inttostr(ypos));
            {$ENDIF}

            if Error then
            begin
              Canvas.Pen.Color := clRed;
              Canvas.Pen.Width := 1;

              l := (cr.Left div 2) * 2;
              if (l mod 4)=0 then o := 2 else o := 0;

              Canvas.MoveTo(l,r.Bottom + o - 1);
              while l < cr.Right do
              begin
                if o = 2 then o := 0 else o := 2;
                Canvas.LineTo(l + 2,r.bottom + o - 1);
                Inc(l,2);
              end;
              // if o = 2 then o := 0 else o := 2;
              // Canvas.LineTo(l + 2,r.Bottom + o - 1);
            end;

            // change to Bidimode
            if DrawStyle and DT_RTLREADING <> DT_RTLREADING then
            begin
              cr.Left := cr.Right;
              cr.Right := r.Right;
            end
            else
              cr.Right := err - cr.Right + cr.Left;

            cr.Bottom := r.Bottom;
            cr.Top := r.Top;
          end
        else
          begin
            cr := r; //reinitialized each time !

            DrawTextEx(Canvas.Handle,PChar(su),WordLenEx,cr,DrawStyle or DT_CALCRECT,nil);

            // preparations for editing purposes
            if (ypos > cr.Top) and (ypos < cr.bottom) and (xpos > w) then {scan charpos here}
            begin
              GetTextExtentExPoint(Canvas.Handle,pChar(su),WordLenEx,xpos-w,@nnfit,nil,nnSize);

              {this will get the character pos of the insertion point}
              if nnfit = WordLen then
                InsPoint := InsPoint + WordLen
              else
                InsPoint := InsPoint + nnfit;
            end;
            // end of preparations for editing purposes

            {Calculated text width}
            WordWidth := cr.Right - cr.Left;
            w := w + WordWidth;

            if DrawStyle and DT_RTLREADING = DT_RTLREADING then
              err := r.Right - XPos - cr.Left
            else
              err := XPos - cr.Left;

            if (XPos - cr.Left >= w - WordWidth) and (err <= w) and Anchor then
            begin
              HotSpot := True;
              if (YPos > cr.Top){ and (YPos < cr.Bottom)} then
              begin
                AnchorVal := LastAnchor;
                AnchorHint := LastAnchorHint;
                MouseInAnchor := True;
              end;
            end;
          end;

          LengthFits := (w < r.Right - r.Left) or (r.Right - r.Left <= WordWidth);

          if not LengthFits and
            ((Length(LineText) > 0) and (LineText[Length(LineText)] <> ' ')) then
            LengthFits := True;

          LineText := LineText + su;

          if LengthFits or not WordWrap then
          begin
            Res := Res + Copy(s,1,WordLen);
            if not LengthFits and Calc and (LineText <> su) then
              s := '';
            Delete(s,1,WordLen);
            if Length(su) >= WordLen then
            begin
              if su[WordLen] = ' ' then
                sw := Canvas.TextWidth(' ')
              else
                sw := 0;
            end
            else
              sw := 0;
          end
          else
          begin
            LineBreak := True;
            w := w - WordWidth;
          end;
        end;
      end;

      TagPos := Pos('<',s);

      if (TagPos = 1) and (Length(s) <= 2) then
        s := '';

      if not LineBreak and (TagPos = 1) and (Length(s) > 2) then
      begin
        if (s[2] = '/') and (Length(s) > 3) then
        begin
          case UpCase(s[3]) of
          'A':begin
                if (not HoverStyle or (Hoverlink = Hyperlinks)) and not Calc then
                begin
                  Canvas.Font.Style := Canvas.Font.Style - [fsUnderline];
                  if Hovercolor <> clNone then
                  begin
                    Canvas.Brush.Color := HvrColor;
                    if HvrColor = clNone then
                      Canvas.Brush.Style := bsClear;
                  end;
                  if HoverFontColor <> clNone then
                    Canvas.Font.Color := HoverFontColor;
                end;

                if not Selected then
                  Canvas.Font.Color := Oldfont.Color;

                Anchor := False;

                if MouseInAnchor then
                begin
                  hr.Bottom := r.Bottom;
                  hr.Right := r.Left + w;
                  if r.Top <> hr.Top then
                  begin
                    hr.Left := r.Left;
                    hr.Top := r.Top;
                  end;

                  HoverRect := hr;
                  MouseLink := HyperLinks;
                  {$IFDEF TMSDEBUG}
                  DbgRect('hotspot anchor '+lastanchor,hr);
                  {$ENDIF}
                  MouseInAnchor := False;
                end;

                if Focuslink = Hyperlinks - 1 then
                begin
                  rr.Right := cr.Left;
                  rr.Bottom := cr.Bottom - ImgAli;
                  rr.Top := rr.Bottom - Canvas.TextHeight('gh');
                  InflateRect(rr,1,0);
                  if not Calc then Canvas.DrawFocusRect(rr);
                end;
              end;
          'E':begin
                if not Calc then
                  Error := False;
              end;
          'B':begin
                if s[4] <> '>' then
                  Canvas.Font.Color := OldFont.Color
                else
                  Canvas.Font.Style := Canvas.Font.Style - [fsBold];
              end;
          'S':begin
                TagChar := UpCase(s[4]);

                if (TagChar = 'T') then  // STRIKE
                begin
                  Canvas.Font.Style := Canvas.Font.Style - [fsStrikeOut];
                end
                else
                if (TagChar = 'U') then  // SUB
                begin
                  isSup := False;
                  isSub := False;
                end
                else
                if (TagChar = 'P') then  // SPAN
                begin
                  Canvas.Brush.Color := BGColor;
                  Canvas.Brush.Style := bsClear;
                end
                else
                if (TagChar = 'H') then
                  isShad := False
                else
                  Canvas.Font.Style := Canvas.Font.Style - [fsStrikeOut];
              end;
          'F':begin
                Canvas.Font.Name := OldFont.Name;
                Canvas.Font.Size := OldFont.Size;
                if not Calc and not Selected then
                begin
                  Canvas.Font.Color := OldFont.Color;
                  Canvas.Brush.Color := BGColor;
                  if BGColor = clNone then
                  begin
                    Canvas.Brush.Style := bsClear;
                  end;
                end;
              end;
          'H':begin
                if not Calc and isHi then
                begin
                  Canvas.Font.Color := hifCol;
                  Canvas.Brush.Color := hibCol;
                  if hibCol = clNone then
                    Canvas.Brush.Style := bsClear;
                end;
              end;
          'I':begin
                Canvas.Font.Style := Canvas.Font.Style - [fsItalic];
              end;
          'L':begin
                LineBreak := True;
              end;
          'P':begin
                LineBreak := True;
                isPara := false;
                if not Calc then
                begin
                  Align := taLeftJustify;
                   Canvas.Brush.Color := ParaColor;
                  if ParaColor = clNone then
                    Canvas.Brush.Style := bsClear;
                end;
              end;
          'U':begin
                if (s[4] <> '>') and (ListIndex > 0) then
                  Dec(Listindex)
                else
                  Canvas.Font.Style := Canvas.Font.Style - [fsUnderline];
              end;
          'O':begin
                if (s[4] <> '>') and (ListIndex > 0) then
                begin
                  Dec(Listindex);
                  if not calc then
                    OList := false;
                end;
              end;
          'R':begin
                EndRotated(Canvas);
              end;
          'Z':Invisible := False;
          end;
        end
        else
        begin
          case Upcase(s[2]) of
          'A':begin
                {only do this when at hover position in xpos,ypos}
                if (FocusLink = HyperLinks) and not Calc then
                begin
                  rr.Left := cr.Left;
                  rr.Top := cr.Top;
                end;

                Inc(HyperLinks);
                if (not HoverStyle or (Hoverlink = HyperLinks)) and not Calc then
                begin
                  Canvas.Font.Style := Canvas.Font.Style + [fsUnderline];
                  if (Hovercolor <> clNone) and not Calc then
                  begin
                    HvrColor := Canvas.Brush.Color;

                    if Canvas.Brush.Style = bsClear then
                      HvrColor := clNone;
                    Canvas.Brush.Color := HoverColor;
                  end;

                  if HoverFontColor <> clNone then
                  begin
                    Canvas.Font.Color := HoverFontColor;
                  end;
                end;

                if not Selected and ((HoverFontColor = clNone) or (HoverLink <> HyperLinks) or not HoverStyle) then
                  Canvas.Font.Color := URLColor;

                TagProp := Uppercase(Copy(s,3,Pos('>',s) - 1));  // <A href="anchor">

                if (VarPos('HREF',TagProp,TagPos) > 0) then
                begin

                  TagProp := Copy(s,3,Pos('>',s) - 1);


                  Prop := Copy(TagProp,TagPos + 4,Length(TagProp));
                  Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
                  Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
                  LastAnchor := Prop;
                  Anchor := True;
                end;

                TagProp := Uppercase(Copy(s,3,Pos('>',s) - 1));  // <A href="anchor">

                if (VarPos('TITLE',TagProp,TagPos) > 0) then
                begin
                  TagProp := Copy(s,3,Pos('>',s) - 1);  // <A href="anchor">
                  Prop := Copy(TagProp,TagPos + 5,Length(TagProp));
                  Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
                  Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
                  LastAnchorHint := Prop;
                end
                else
                  LastAnchorHint := '';

{
                TagProp := Copy(s,3,Pos('>',s) - 1);  // <A href="anchor">
                Prop := Copy(TagProp,Pos('"',TagProp) + 1,Length(TagProp));
                Prop := Copy(Prop,1,Pos('"',Prop) - 1);
                LastAnchor := Prop;
                Anchor := True;
}

                hr.Left := w;
                hr.Top := r.Top;
              end;
          'B':begin
                TagChar := Upcase(s[3]);
                if TagChar = '>' then  // <B> tag
                  Canvas.Font.Style := Canvas.Font.Style + [fsBold]
                else
                  if TagChar = 'R' then // <BR> tag
                  begin
                    LineBreak := true;
                    StripVal := StripVal + #13;
                  end
                  else
                  begin
                    if TagChar = 'L' then // <BLINK> tag
                    begin
                      if not Blink then Canvas.Font.Color := BlnkColor;
                    end
                    else
                    if TagChar = 'O' then  // <BODY ... >
                    begin
                      Res := Res + Copy(s,1,pos('>',s));
                      TagProp := Uppercase(Copy(s,6,pos('>',s)-1));

                      if (Pos('FONT-FAMILY',TagProp) > 0) then
                      begin
                        Prop := Copy(TagProp,Pos('FONT-FAMILY',TagProp)+11,Length(TagProp));
                        Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop)+1,Length(prop));
                        Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop)-1);
                        Canvas.Font.Name := Prop;
                      end;

                      if (Pos('FONT-SIZE',TagProp) > 0) then
                      begin
                        Prop := Copy(TagProp,Pos('FONT-SIZE',TagProp)+10,Length(TagProp));
                        fc := FirstChar(';''"', Prop);
                        if fc > 0 then
                          Prop := Copy(Prop,1,fc-1);

                        if pos('PT', Prop) > 0 then
                          Prop := Copy(Prop, 1, Length(Prop) - 2);

                        Val(Prop, fc, err);
                        if err = 0 then
                          Canvas.Font.Size := fc;

                        th := Canvas.TextHeight(su);

                        if th > h then
                          h := th;
                      end;

                      if (Pos('FONT-STYLE',TagProp) > 0) then
                      begin
                        Prop := Copy(TagProp,Pos('FONT-STYLE',TagProp)+11,Length(TagProp));
                        fc := FirstChar(';''"', Prop);
                        if fc > 0 then
                          Prop := Copy(Prop,1,fc - 1);

                        if Prop = 'BOLD' then
                          Canvas.Font.Style := [fsBold];
                        if Prop = 'ITALIC' then
                          Canvas.Font.Style := [fsItalic];
                      end;

                    end; //end of background

                      if (Pos('BGCOLOR',TagProp)>0) then
                      begin
                        Prop := Copy(TagProp,Pos('BGCOLOR',TagProp) + 7,Length(TagProp));
                        Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
                        Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
                        if not Calc then
                        begin
                          if Pos('CL',Prop) > 0 then
                            Canvas.Brush.color := Text2Color(AnsiLowerCase(Prop));
                          if Pos('#',Prop) > 0 then
                            Canvas.Brush.color := Hex2Color(Prop);

                          if not Calc then
                          begin
                            BGColor := Canvas.Brush.Color;
                            Pencolor := Canvas.Pen.Color;
                            Canvas.Pen.color := BGColor;
                            Canvas.Rectangle(fr.Left,fr.Top,fr.Right,fr.Bottom);
                            Canvas.Pen.Color := PenColor;
                          end;
                        end;
                    end;
                  end;
                end;
          'E':begin
                if not Calc then
                  Error := True;
              end;
          'C':begin
                { control here }
                { <CONTROL type="EDIT" width="125" ID="name" VALUE="" MAXLEN=""> }

                TagProp := Copy(s,9,pos('>',s)-1);
                ParseControl(TagProp,ControlType,ControlID,ControlValue,ControlWidth,ControlHeight,ControlProp,ControlLen);

                ch := 25;

                if ControlHeight <> '' then
                begin
                  val(ControlHeight,ch,err);
                  if err <> 0 then
                    ch := 25;
                end;

                ch := Round(ch * scale);

                if ControlWidth <> '' then
                begin
                  val(ControlWidth,Indent,err);

                  Indent := Round(Indent * scale);

                  if err = 0 then
                  begin
                    IMGSize.x := Indent;
                    IMGSize.y := Canvas.TextHeight('gh') + 10; //changed
                  end;

                  // standard Windows control size
                  if (ControlType = 'CHECK') or (ControlType = 'RADIO') then
                    h := 16
                  else
                    h := 23;

                  if (ch > Round(25*scale)) then
                    h := ch - 2
                  else
                    h := Round(scale * h);

                  if not Calc then
                  begin
                    if (ControlType = 'EDIT') or (ControlType = 'PASSWORD') or (ControlType = 'MASK') then
                    begin
                      if UseWinXp then
                        Canvas.Pen.Color := $B99D7F
                      else
                        Canvas.Pen.Color := clGray;

                      Canvas.Brush.Color := clWhite;
                      Canvas.Brush.Style := bsSolid;
                      Canvas.Rectangle(cr.Left ,cr.Bottom - h + 4,cr.Left + Indent, cr.Bottom + 2);
                      Canvas.Brush.Style := bsClear;

                      ir := Rect(cr.Left + 2,cr.Bottom - h + 4,cr.Left + Indent, cr.Bottom);

                      if (ControlType = 'PASSWORD') then
                        DrawText(Canvas.Handle,PChar(ConvertToPassword(ControlValue)),Length(ControlValue),ir,DT_LEFT  or DT_VCENTER or DT_SINGLELINE)
                      else
                        DrawText(Canvas.Handle,PChar(ControlValue),Length(ControlValue),ir,DT_LEFT or DT_VCENTER or DT_SINGLELINE);
                    end;

                    if ControlType = 'COMBO' then
                    begin
                      if UseWinXp then
                        Canvas.Pen.Color := $B99D7F
                      else
                        Canvas.Pen.Color := clGray;

                      Canvas.Brush.Color := clWhite;
                      Canvas.Brush.Style := bsSolid;
                      Canvas.Rectangle(cr.Left ,cr.Bottom - h + 4,cr.Left + Indent, cr.Bottom + 1);

                      Canvas.Brush.Style := bsClear;
                      ir := Rect(cr.Left + 4,cr.Bottom - h + 4,cr.Left + Indent - Round(18*scale), cr.Bottom);
                      DrawText(Canvas.Handle,PChar(ControlValue),Length(ControlValue),ir,DT_LEFT or DT_SINGLELINE or DT_VCENTER);

                      ir := Rect(cr.Left + Indent - Round(18*scale),cr.Bottom - h + 5,cr.Left + Indent - 1, cr.Bottom + 1);

                      DrawFrameControl(Canvas.Handle,ir,DFC_SCROLL,DFCS_SCROLLCOMBOBOX);
                    end;

                    if ControlType = 'CHECK' then
                    begin
                      IMGSize.x := Round(16 * scale);
                      IMGSize.y := IMGSize.x;
                      Indent := IMGSize.x;

                      if UseWinXp then
                        Canvas.Pen.Color := $B99D7F
                      else
                        Canvas.Pen.Color := clGray;

                      Canvas.Brush.Style := bsClear;
                      ir := Rect(cr.Left + 2,cr.Bottom - (IMGSize.y - 1),cr.Left + (ImgSize.X - 1), cr.Bottom);

                      FHot := False;

                      if Uppercase(ControlValue) = 'TRUE' then
                        DrawFrameControl(Canvas.Handle,ir,DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_FLAT or DFCS_CHECKED)
                      else
                        DrawFrameControl(Canvas.Handle,ir,DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_FLAT);
                    end;

                    if ControlType = 'RADIO' then
                    begin
                      IMGSize.x := Round(16*scale);
                      IMGSize.y := Round(16*scale);
                      Indent := Round(16*scale);

                      if UseWinXp then
                        Canvas.Pen.Color := $B99D7F
                      else
                        Canvas.Pen.Color := clGray;
                      Canvas.Brush.Style := bsClear;
                      ir := Rect(cr.Left + 2,cr.Bottom - Round(14*scale),cr.Left + Round(14*scale), cr.Bottom);

                      FHot := (XPos > cr.Left - 2) and (XPos < cr.Left + IMGSize.x - 2) and
                              (YPos < cr.Bottom - 2) and (YPos > cr.Bottom - IMGSize.y - 2);

                      FHot := false;

                      if Uppercase(ControlValue) = 'TRUE' then
                        DrawFrameControl(Canvas.Handle,ir,DFC_BUTTON, DFCS_BUTTONRADIO or DFCS_FLAT or DFCS_CHECKED)
                      else
                        DrawFrameControl(Canvas.Handle,ir,DFC_BUTTON, DFCS_BUTTONRADIO or DFCS_FLAT);
                    end;

                    if ControlType = 'BUTTON' then
                    begin
                      IMGSize.y := Round(24*scale);

                      if UseWinXp then
                        Canvas.Pen.Color := $B99D7F
                      else
                        Canvas.Pen.Color := clGray;

                      Canvas.Brush.Style := bsClear;

                      ir := Rect(cr.Left + 2,cr.Bottom - ch,cr.Left + Indent -2, cr.Bottom);

                      FHot := (XPos > cr.Left -2) and (XPos < cr.Left + IMGSize.x - 2) and
                         (YPos < cr.Bottom - 2) and (YPos > cr.Bottom - IMGSize.y - 2);

                      if FHot and Down and (DownID = ControlID) then
                        DrawFrameControl(Canvas.Handle,ir,DFC_BUTTON, DFCS_BUTTONPUSH or DFCS_PUSHED)
                      else
                        DrawFrameControl(Canvas.Handle,ir,DFC_BUTTON, DFCS_BUTTONPUSH);

                      InflateRect(ir,-2,-2);
                      DrawText(Canvas.Handle,PChar(ControlValue),Length(ControlValue),ir,DT_CENTER or DT_VCENTER or DT_SINGLELINE);
                    end;
                  end;

                  if (ControlType = 'BUTTON') then
                    IMGSize.y := Round(25*scale);

                  if (ControlType = 'COMBO') then
                     IMGSize.y := Round(25*scale);

                  if (ControlType = 'CHECK') or (ControlType = 'RADIO') then
                  begin
                    IMGSize.x := Round(16 * scale * ResFactor);
                    IMGSize.y := Round(16 * scale);
                  end;

                  if ((XPos - r.Left > w) and (XPos - r.Left < w + IMGSize.x) and
                     (YPos > cr.Top) and (YPos < cr.Top + IMGSize.Y)) or
                     (CheckHotSpot and (CID = ControlID)) then
                  begin
                    ImageHotSpot := True;
                    AnchorVal := 'ctrl';
                    AltImg := ImgIdx;

//                    ir := cr;
//                    ir.Right := cr.Left + IMGSize.x;

                    ir.Left := r.left + w;
                    ir.Right := ir.Left + ImgSize.X;
                    ir.Top := cR.Top;
                    ir.Bottom := cr.top + ImgSize.Y;

                    ControlRect := ir;
                    CV := ControlValue;
                    CID := ControlID;
                    CT := ControlType;
                  end;

                    if (w + IMGSize.x > r.Right-r.Left) and
                       (IMGSize.x < r.Right - r.Left) then
                    begin
                      ImgBreak := True;
                    end
                    else
                      begin
                        w := w + IMGSize.x;
                        cr.left := cr.left + IMGSize.x;
                        if IMGSize.y > h then
                          h := IMGSize.y;
                      end;

///                  cr.left := fr.left + Indent;
                end;
              end;
          'H':begin
                case Upcase(s[3]) of
                'R':
                  begin
                    LineBreak := True;
                    if not Calc then
                    begin
                      Pencolor := Canvas.Pen.color;
                      Canvas.Pen.color:=clblack;
                      Canvas.MoveTo(r.left,cr.bottom+1);
                      Canvas.Lineto(r.right,cr.bottom+1);
                      Canvas.pen.color:=pencolor;
                    end;
                  end;
                  'I':
                  begin
                    if not Calc then
                    begin
                      isHi := true;
                      hifCol := Canvas.Font.Color;
                      hibCol := Canvas.Brush.Color;
                      if Canvas.Brush.Style = bsClear then
                        hibCol := clNone;

                      Canvas.Brush.Color := HTMLHighLightColor;
                      Canvas.Font.Color := HTMLHighLightTextColor;
                    end;
                  end;
                end;
              end;
          'I':begin
                TagChar := Upcase(s[3]);

                if TagChar = '>' then // <I> tag
                  Canvas.Font.Style := Canvas.Font.Style + [fsItalic]
                else
                if TagChar = 'N' then  // <IND> tag
                begin
                  TagProp := Copy(s,3,pos('>',s)-1);

                  Prop := Copy(TagProp,ipos('x',TagProp)+2,Length(TagProp));
                  Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop)+1,Length(prop));
                  Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop)-1);

                  val(Prop,indent,err);
                  if err = 0 then
                  begin
                    if Indent > w then
                     begin
                       w := Indent;
                       cr.left := fr.left + Indent;
                     end;
                  end;
                end
                else
                  if TagChar = 'M' then
                  begin
                    imgfloat := fsNone;

                    flr := cr;

                    inc(ImgIdx);

                    //oldfont.color:=Canvas.font.color;
                    TagProp := Uppercase(Copy(s,3,pos('>',s) - 1));
                    Prop := Copy(TagProp,Pos('SRC',TagProp) + 4,Length(TagProp));
                    Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
                    Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);

                    if (Pos('ALT',TagProp) > 0) and (AltImg = ImgIdx) then
                    begin
                      Prop := Copy(TagProp,Pos('ALT',TagProp) + 4,Length(TagProp));
                      Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
                      Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);
                    end;

                    TagWidth := 0;
                    TagHeight := 0;

                    if Pos('WIDTH',TagProp) > 0 then
                    begin
                      Tagp := Copy(TagProp,Pos('WIDTH',TagProp) + 6,Length(TagProp));
                      Tagp := Copy(Tagp,Pos(WL_ATTR_DELIM,tagp) + 1,Length(Tagp));
                      Tagp := Copy(Tagp,1,Pos(WL_ATTR_DELIM,tagp) - 1);
                      Val(Tagp,TagWidth,Err);
                      TagWidth := Round(TagWidth * scale);
                    end;

                    if Pos('HEIGHT',TagProp) > 0 then
                    begin
                      Tagp := Copy(TagProp,ipos('HEIGHT',TagProp) + 7,Length(TagProp));
                      Tagp := Copy(Tagp,pos(WL_ATTR_DELIM,Tagp) + 1,Length(Tagp));
                      Tagp := Copy(Tagp,1,pos(WL_ATTR_DELIM,Tagp) - 1);
                      Val(Tagp,TagHeight,Err);
                      TagHeight := Round(TagHeight * scale);
                    end;

                    if Pos('ALIGN',TagProp) > 0 then
                    begin
                      Tagp := Copy(TagProp,ipos('ALIGN',TagProp) + 6,Length(TagProp));
                      Tagp := Copy(Tagp,pos('"',Tagp) + 1,Length(Tagp));
                      Tagp := Copy(Tagp,1,pos('"',Tagp) - 1);
                    end;

                    if Pos('FLOAT',TagProp) > 0 then
                    begin
                      Tagp := Copy(TagProp,ipos('FLOAT',TagProp) + 6,Length(TagProp));
                      Tagp := Copy(Tagp,pos('"',Tagp) + 1,Length(Tagp));
                      Tagp := Copy(Tagp,1,pos('"',Tagp) - 1);

                      if TagP = 'LEFT' then
                      begin
                        imgfloat := fsLeft;
                        ir.left := fr.Left;
                      end;

                      if TagP = 'RIGHT' then
                        imgfloat := fsRight;
                    end;

                    IMGSize.x := 0;
                    IMGSize.y := 0;

                    if Pos('IDX:',Prop) > 0 then
                    begin
                      Delete(Prop,1,4);
                      if Assigned(FImages) and (IStrToInt(Prop) < FImages.Count) then
                      begin
                        //IMGSize.x := Round(FImages.Width * scale);
                        //IMGSize.y := Round(FImages.Height * scale);
                        IMGSize.x := FImages.Width;
                        IMGSize.y := FImages.Height;

                        if not Calc and not Print then
                          FImages.Draw(Canvas,flr.Left,flr.Top,IStrToInt(Prop),True);
                      end;
                    end;

                    if Pos('SSYS:',Prop) > 0 then
                    begin
                      Delete(Prop,1,5);
                      IMGSize := SysImage(Canvas,flr.Left,flr.Top,Prop,False,not Calc,Print,ResFactor);

                      //IMGSize.x := Round(IMGSize.X * scale);
                      //IMGSize.y := Round(IMGSize.Y * scale);
                    end;

                    if Pos('LSYS:',Prop) > 0 then
                    begin
                      Delete(Prop,1,5);
                      IMGsize := SysImage(Canvas,flr.Left,flr.Top,Prop,True,not Calc,Print,ResFactor);

                      //IMGSize.x := Round(IMGSize.X * scale);
                      //IMGSize.y := Round(IMGSize.Y * scale);
                    end;


                    if (XPos - r.Left > w) and (XPos - r.Left < w + IMGSize.x) and
                       (YPos > cr.Top) and (YPos < cr.Top + IMGSize.Y) and Anchor then
                    begin
                      ImageHotSpot := True;
                      AnchorVal := LastAnchor;
                      AnchorHint := LastAnchorHint;
                      AltImg := ImgIdx;
                    end;

                    if Print then
                    begin
                      //IMGSize.x := Round(IMGSize.x * ResFactor);
                      //IMGSize.y := Round(IMGSize.y * ResFactor);
                      {$IFDEF TMSDEBUG}
                      DbgPoint('bmp : ',point(IMGSize.x,IMGSize.y));
                      {$ENDIF}
                    end;

                    if (w + IMGSize.x > r.Right-r.Left) and
                       (IMGSize.x < r.Right - r.Left) then
                    begin
                      ImgBreak := True;
                    end
                    else
                      begin
                        w := w + IMGSize.x + 2;
                        cr.left := cr.left + IMGSize.x + 2;
                        if IMGSize.y > h then
                          h := IMGSize.y;
                      end;

                    txth := Canvas.TextHeight('gh');

                    if Pos('ALIGN',TagProp) > 0 then
                    begin
                      if Pos('"TOP',TagProp) > 0 then
                      begin
                        if h > txth then
                          ImgAli := h - txth
                        else
                          ImgAli := 0;
                      end
                      else
                      begin
                        if Pos('"MIDDLE',TagProp) > 0 then
                        begin
                          if h > txth then
                            ImgAli := (h - txth) shr 1
                          else
                            ImgAli := 0;
                        end;
                      end;
                    end;

                    if (Pos('WRAP',TagProp) > 0) then
                    begin
                      h := txth;
                      ImgAli := 0;
                    end;

                    if imgfloat = fsLeft then
                    begin
                      if h > txth then
                        ImgAli := h - txth
                      else
                        ImgAli := 0;
                      floatrect.left := imgw + 2;
                      floatrect.bottom := imgh;
                      if not calc then
                        h := txth;
                    end;

                    if imgfloat = fsRight then
                    begin
                      if h > txth then
                        ImgAli := h - txth
                      else
                        ImgAli := 0;
                      floatrect.right := imgw + 2;
                      floatrect.bottom := imgh;
                      if not calc then
                        h := txth;
                      w := fr.Right;
                    end;

                  end;
                end;
          'L':begin
                w := w + 12 * ListIndex;
                if Linkbreak then
                  Imgbreak := True
                else
                  Linkbreak := True;

                cr.left := cr.left + 12 * (ListIndex - 1);

                if not calc and not Invisible then
                begin
                  Prop := Canvas.Font.Name;
                  Canvas.Font.Name := 'Symbol';

                  if not OList then
                  begin
                    if Odd(ListIndex) then
                      DrawText(Canvas.Handle,'',1,cr,0)
                    else
                      DrawText(Canvas.Handle,'o',1,cr,0);
                  end
                  else
                  begin
                    OListIndexStr := IntToStr(OListIndex)+'. ';
                    Inc(OListIndex);

                    DrawText(Canvas.Handle,PChar(OListIndexStr),Length(OListIndexStr),cr,0);
                    cr.Left := cr.Left + Canvas.TextWidth(OListIndexStr) - 12;
                  end;

                  Canvas.Font.Name:=prop;
                end;
                cr.Left := cr.Left + 12;
              end;
          'U':begin
                if s[3] <> '>' then
                begin
                  if not Calc then
                  begin
                    Inc(ListIndex);
                    OList := false;
                  end;
                  if w > 0 then
                    LineBreak := True;
                end
                else
                  Canvas.Font.Style := Canvas.Font.Style + [fsUnderline];
              end;
          'O':begin
                TagChar := Upcase(s[3]);
                if TagChar = 'L' then  // <OFS> tag
                begin
                  if not Calc then
                    Inc(ListIndex);
                  if w > 0 then
                    LineBreak := true;
                  OList := true;
                  OListIndex := 1;
                end;
              end;

          'P':begin
                //if calc then
                //  Linebreak := true
                //else
                begin
                  if (VarPos('>',s,TagPos)>0) then
                  begin
                    TagProp := Uppercase(Copy(s,3,TagPos-1));

                    if VarPos('ALIGN',TagProp,TagPos) > 0 then
                    begin
                      Prop := Copy(TagProp,TagPos+5,Length(TagProp));
                      Prop := Copy(Prop,Pos(WL_ATTR_DELIM,prop)+1,Length(Prop));
                      Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,prop)-1);

                      if Pos('RIGHT',Prop) > 0 then Align := taRightJustify;
                      if Pos('LEFT',Prop) > 0 then Align := taLeftJustify;
                      if Pos('CENTER',Prop) > 0 then Align := taCenter;
                    end;

                    if VarPos('INDENT',TagProp,TagPos) > 0 then
                    begin
                      Prop := Copy(TagProp,TagPos+6,Length(TagProp));
                      Prop := Copy(Prop,Pos(WL_ATTR_DELIM,prop)+1,Length(Prop));
                      Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,prop)-1);
                      PIndent := IStrToInt(Prop);
                    end;

                    if VarPos('FLOAT',TagProp,TagPos) > 0 then
                    begin
                      Prop := Copy(TagProp,TagPos+5,Length(TagProp));
                      Prop := Copy(Prop,Pos('"',prop)+1,Length(Prop));
                      Prop := Copy(Prop,1,Pos('"',prop)-1);

                      if Prop = 'LEFT' then
                        txtfloat := fsLeft;

                      if Prop = 'RIGHT' then
                        txtfloat := fsRight;

                      floatpt := Point(r.Left, r.Top);
                    end;

                    if VarPos('BGCOLOR',TagProp,TagPos) > 0 then
                    begin
                      Prop := Copy(TagProp,TagPos + 5,Length(TagProp));
                      Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop) + 1,Length(Prop));
                      Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,Prop) - 1);

                      NewColor := clNone;

                      if Length(Prop) > 0 then
                      begin
                        if Prop[1] = '#' then
                          NewColor := Hex2Color(Prop)
                        else
                          NewColor := Text2Color(AnsiLowerCase(prop));
                      end;

                      if not Calc then
                      begin
                        isPara := True;
                        paracolor := Canvas.Brush.Color;
                        if Canvas.Brush.Style = bsClear then ParaColor := clNone;
                        Canvas.Brush.color := NewColor;
                        PenColor:=Canvas.Pen.Color;
                        Canvas.Pen.Color := Newcolor;
                        Canvas.Rectangle(fr.left,r.top,fr.right,r.bottom);
                      end;
                    end;
                  end;
                end;
            end;
        'F':begin
              if (VarPos('>',s,TagPos)>0) then
              begin
                TagProp := UpperCase(Copy(s,6,TagPos-6));

                if (VarPos('FACE',TagProp,TagPos) > 0) then
                begin
                  Prop := Copy(TagProp,TagPos+4,Length(TagProp));
                  Prop := Copy(prop,pos(WL_ATTR_DELIM,prop)+1,Length(prop));
                  Prop := Copy(prop,1,pos(WL_ATTR_DELIM,prop)-1);
                  Canvas.Font.Name := Prop;
                end;

                if (VarPos(' COLOR',TagProp,TagPos) > 0) and not Selected then
                begin
                  Prop := Copy(TagProp,TagPos+6,Length(TagProp));
                  Prop := Copy(Prop,Pos(WL_ATTR_DELIM,prop)+1,Length(prop));
                  Prop := Copy(Prop,1,Pos(WL_ATTR_DELIM,prop)-1);
                  //oldfont.color:=Canvas.font.color;

                  if Length(Prop) > 0 then
                  begin
                    if Prop[1] = '#' then
                      Canvas.font.color := Hex2Color(Prop)
                    else
                      Canvas.Font.Color := Text2Color(AnsiLowerCase(prop));
                  end;

                end;

                if (VarPos('BGCOLOR',TagProp,TagPos)>0) and not Calc and not Selected then
                begin
                  Prop := Copy(TagProp,TagPos+7,Length(TagProp));
                  Prop := Copy(prop,pos(WL_ATTR_DELIM,prop)+1,Length(prop));
                  Prop := Copy(prop,1,pos(WL_ATTR_DELIM,prop)-1);
                  BGColor := Canvas.Brush.Color;

                  if Canvas.Brush.Style = bsClear then
                    bgcolor := clNone;

                  if Length(Prop) > 0 then
                  begin
                    if Prop[1] = '#' then
                      Canvas.Brush.Color := Hex2Color(Prop)
                    else
                      Canvas.Brush.Color := Text2Color(AnsiLowerCase(prop));
                  end;
                end;

                if (VarPos('STYLE',TagProp,TagPos)>0) then
                begin
                  Prop := Copy(TagProp,TagPos+5,Length(TagProp));
                  Prop := Copy(Prop,Pos('=',Prop)+1,Length(Prop));
                  Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop)+1,Length(Prop));
                  // font-size:14pt
                  if Pos('FONT-SIZE:', Uppercase(Prop)) > 0 then
                  begin
                    Delete(Prop, 1, 10);

                    if pos('PT', UpperCase(Prop)) > 0 then
                    begin
                      Prop := Trim(Copy(Prop,1, Pos('PT',Uppercase(Prop))-1));
                      Canvas.Font.Size := IStrToInt(Prop);
                    end;
                  end;
                end
                else
                if (VarPos('SIZE',TagProp,TagPos)>0) then
                begin
                  Prop := Copy(TagProp,TagPos+4,Length(TagProp));
                  Prop := Copy(Prop,Pos('=',Prop)+1,Length(Prop));
                  Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop)+1,Length(Prop));

                  case IStrToInt(Prop) of
                  1:Canvas.Font.Size := 8;
                  2:Canvas.Font.Size := 10;
                  3:Canvas.Font.Size := 12;
                  4:Canvas.Font.Size := 14;
                  5:Canvas.Font.Size := 16;
                  else
                    Canvas.Font.Size := IStrToInt(Prop);
                  end;
                end;
              end;
            end;
        'S':begin
              TagChar := Upcase(s[3]);

              if TagChar = '>' then
                Canvas.Font.Style := Canvas.font.Style + [fsStrikeOut]
              else
              begin
                if TagChar = 'T' then
                begin
                  Canvas.Font.Style := Canvas.font.Style + [fsStrikeOut]
                end
                else
                if TagChar = 'P' then
                begin
                  if (VarPos('>',s,TagPos)>0) then
                  begin

                    TagProp := UpperCase(Copy(s,6,TagPos-6));
                    if (VarPos('STYLE',TagProp,TagPos)>0) then
                    begin
                      Prop := Copy(TagProp,TagPos+5,Length(TagProp));
                      Prop := Copy(Prop,Pos('=',Prop)+1,Length(Prop));
                      Prop := Copy(Prop,Pos(WL_ATTR_DELIM,Prop)+1,Length(Prop));
                      // font-size:14pt
                      if Pos('BACKGROUND-COLOR:', Uppercase(Prop)) > 0 then
                      begin
                        Delete(Prop, 1, 17);

                        if pos('#', UpperCase(Prop)) = 1 then
                        begin
                          BGColor := Canvas.Brush.Color;
                          Canvas.Brush.Color := Hex2Color(Prop);
                        end;
                      end;
                    end;
                  end;
                end
                else
                if TagChar = 'H' then
                  isShad := True
                else
                begin
                  if ipos('<SUB>',s)=1 then
                    isSub := True
                  else
                    if ipos('<SUP>',s)=1 then
                      isSup := True;
                end;
              end;
            end;
        'R':begin
              TagProp := Copy(s,3,pos('>',s)-1);
              prop := Copy(TagProp,ipos('a',TagProp)+2,Length(TagProp));
              prop := Copy(prop,pos(WL_ATTR_DELIM,prop)+1,Length(prop));
              prop := Copy(prop,1,pos(WL_ATTR_DELIM,prop)-1);
              Val(prop,Indent,err);
              StartRotated(Canvas,indent);
            end;
        'Z':Invisible := True;
        end;
      end;

      if (VarPos('>',s,TagPos) > 0) and not ImgBreak then
      begin
        Res := Res + Copy(s,1,TagPos);
        Delete(s,1,TagPos);
      end
      else
      begin
        if not Imgbreak then
          Delete(s,1,Length(s));
      end;
    end;

  end;

    w := w - sw;

    if w > xsize then
      xsize := w;

    if (FocusLink = Hyperlinks-1) and Anchor and not Calc then
    begin
      rr.Right := cr.Left;
      rr.Bottom := cr.Bottom;
      InflateRect(rr,1,0);
      if not Calc then
        Canvas.DrawFocusRect(rr);
      rr.Left := r.Left + 1;
      rr.Top := rr.Bottom;
    end;

    Result := Res;
  end;

  function DrawTextBidimodeFlags(DrawStyle: integer; BidiMode: TBidiMode): integer;
  var
    bool : boolean;

  begin
    bool := SysLocale.MiddleEast and (BiDiMode = bdRightToLeft);
    Result := DrawStyle;
    // do not change center alignment
    if bool then
      if Result and DT_RIGHT = DT_RIGHT then
        Result := Result and not DT_RIGHT { removing DT_RIGHT, makes it DT_LEFT }
      else
      if not (Result and DT_CENTER = DT_CENTER) then
        Result := Result or DT_RIGHT;

    bool := SysLocale.MiddleEast and (BiDiMode <> bdLeftToRight);

    if bool then
      Result := Result or DT_RTLREADING;
  end;

begin
  scale := GetDeviceCaps(Canvas.Handle, LOGPIXELSY) / 96;

  Anchor := False;
  Error := False;
  OrgPenColor := Canvas.Pen.Color;
  OrgPenWidth := Canvas.Pen.Width;
  OldFont := TFont.Create;
  OldFont.Assign(Canvas.Font);
  DrawFont := TFont.Create;
  DrawFont.Assign(Canvas.Font);
  CalcFont := TFont.Create;
  CalcFont.Assign(Canvas.Font);
  OldDrawfont := TFont.Create;
  OldDrawFont.Assign(Canvas.Font);
  OldCalcFont := TFont.Create;
  OldCalcFont.Assign(Canvas.Font);
  BlnkColor := Canvas.Brush.color;
  Canvas.Brush.Color := clNone;
  BGColor := clNone;
  ParaColor := clNone;
  isPara := False;
  isShad := False;
  Invisible := False;
  txtfloat := fsNone;
  floatrect := Rect(0,0,0,0);

  // Control param initialization
  ControlRect := Rect(0,0,0,0);
  CV := '';
  CT := '';
  if not CheckHotSpot then
    CID := '';

  Result := False;

  fr.Left := fr.Left + 1;  {required to add offset for DrawText problem with first capital W letter}
  r := fr;

  Align := taLeftJustify;
  PIndent := 0;

  XSize := 0;
  YSize := 0;
  HyperLinks := 0;
  HlCount := 0;
  ListIndex := 0;
  LiCount := 0;
  StripVal := '';
  FocusAnchor := '';
  AnchorHint := '';
  MouseLink := -1;
  MouseInAnchor := False;

  if ResFactor < 1 then
    ResFactor := 1;

  ImgIdx := 0;
  AltImg := -1;

  SetBKMode(Canvas.Handle,TRANSPARENT);

  DrawStyle := DT_LEFT or DT_SINGLELINE or DT_EXTERNALLEADING or DT_BOTTOM  or DT_EXPANDTABS {or DT_NOPREFIX};

  if not WordWrap then
    DrawStyle := DrawStyle or DT_END_ELLIPSIS;

  if Pos('&',s) > 0 then
  begin
    repeat
      Foundtag := False;
      s := UnFixMarkup(s, true);
      if TagReplacestring('&amp;','&&',s) then
        Foundtag := True;
    until not Foundtag;
  end;

  s := DBTagStrip(s);
  s := UnicodeTagStrip(s);
  s := CRLFStrip(s,WL_KEEP_LINEBREAKS);
  isHi := False;

  InsPoint := 0;

  while Length(s) > 0 do
  begin
    // calculate part of the HTML text fitting on the next line
    Oldfont.Assign(OldCalcFont);
    Canvas.Font.Assign(CalcFont);
    Oldanchor := Anchor;
    OldAnchorVal := LastAnchor;
    OldAnchorHint := LastAnchorHint;
    suph := 0;
    subh := 0;
    imgali := 0;
    isSup := False;
    isSub := False;

    HtmlHeight := Canvas.TextHeight('gh');
    txtHeight := HTMLHeight;

    OldImgIdx := ImgIdx;
    txtfloatbefore := txtfloat;

    su := HTMLDrawLine(Canvas,s,r,True,HtmlWidth,HtmlHeight,subh,suph,imgali,Align,PIndent,XPos,YPos,HotSpot,ImageHotSpot,floatrect,txtfloat);

    Anchor := OldAnchor;
    LastAnchor := OldAnchorVal;
    LastAnchorHint := OldAnchorHint;

    CalcFont.Assign(Canvas.Font);
    OldCalcFont.Assign(OldFont);

    dr := r;

    case Align of
    taCenter:if (r.right - r.left - htmlwidth > 0) then
               dr.left := r.left+((r.right - r.left - htmlwidth) shr 1);
    taRightJustify:if r.right - htmlwidth > r.left then
                       dr.left := r.right - htmlwidth;
    end;

    dr.Left := dr.Left + PIndent;
    dr.Bottom := dr.Top + HtmlHeight + Subh + Suph;

    if not CheckHeight then
    begin
      OldFont.Assign(OldDrawFont);
      Canvas.Font.Assign(DrawFont);

      HyperLinks := HlCount;
      ListIndex := LiCount;
      ImgIdx := OldImgIdx;

      HTMLDrawLine(Canvas,su,dr,CheckHotSpot,HtmlWidth,HtmlHeight,subh,suph,ImgAli,Align,PIndent,XPos,YPos,HotSpot,ImageHotspot,floatrect,txtfloat);

      HlCount := HyperLinks;
      LiCount := ListIndex;

      r.left := fr.Left + floatrect.left;

      r.right := fr.Right - floatrect.right;

      if dr.Top > fr.Top + floatrect.bottom then
      begin
        floatrect.Left := 0;
        floatrect.Right := 0;
        r.Left := fr.Left;
      end;

      if (HotSpot and
         (YPos > dr.Bottom - ImgAli - Canvas.TextHeight('gh')) and
         (YPos < dr.Bottom - ImgAli)) or ImageHotSpot then
        Result := True;

      DrawFont.Assign(Canvas.Font);
      OldDrawFont.Assign(OldFont);
    end;

    if (txtfloat <> fsNone) and (txtfloat <> txtfloatbefore) then
    begin
      r.Left := floatpt.x;
      r.Top := floatpt.y;
      txtfloat := fsNone;
    end
    else
    begin
      r.top := r.top + HtmlHeight + subh + suph;
      ysize := ysize + HtmlHeight + subh + suph;
    end;


    // do not draw below bottom
    if (r.top + txtheight > r.bottom) and not CheckHeight then
      s := '';
  end;

  if (ysize = 0) then
    ysize := Canvas.TextHeight('gh');

  InsPoint := InsPoint shr 1;

  Canvas.Pen.Color := OrgPenColor;
  Canvas.Pen.Width := OrgPenWidth;

  Canvas.Brush.Color := BlnkColor;
  Canvas.Font.Assign(OldFont);
  OldFont.Free;
  DrawFont.Free;
  CalcFont.Free;
  OldDrawfont.Free;
  OldCalcfont.Free;
end;
{$WARNINGS ON}

function IPosFrom(su,s:string;frm:integer):Integer;
var
  i:Integer;
begin
  i := Pos(UpperCase(su),UpperCase(s));
  if i > frm then
    Result := i
  else
    Result := 0;
end;

function HTMLStrip(s:string):string;
var
  Tpf, Tpn: integer;
begin
  Result := '';
  // replace line breaks by linefeeds
  while (pos('<BR>',uppercase(s))>0) do s := StringReplace(s,'<BR>',chr(13)+chr(10),[rfIgnoreCase]);
  while (pos('<HR>',uppercase(s))>0) do s := StringReplace(s,'<HR>',chr(13)+chr(10),[rfIgnoreCase]);

  {remove all other tags}
  while (VarPos('<',s,Tpf) > 0) do
  begin
    Result := Result + Copy(s, 1, Tpf - 1);
    if (VarPos('>',s, Tpn)>0) then
      Delete(s,1,Tpn)
    else
    begin
      Delete(s,1,Tpf - 1);
      Break;
    end;
  end;
  Result := Result + s;
end;


end.
