{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2019                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}
unit WEBLib.HTTP;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, System.SysUtils, Winapi.Wininet, Vcl.Graphics, Winapi.Windows,
  Vcl.Imaging.PNGImage, Vcl.Imaging.JPEG, Vcl.Imaging.GIFImg;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils
  {$IFDEF WINDOWS}
  , WinInet, Windows
  {$ENDIF}
  , Graphics;
  {$ENDIF}

type
  TDownLoadThread = class(TThread)
  strict private
    FPicture: TPicture;
    FStream: TMemoryStream;
    FURL: string;
  strict protected
    procedure Download;
    procedure DownLoadError(const AError: string);
    procedure DownloadProgress(APosition, ASize: Integer);
    procedure Execute; override;
    procedure UpdatePicture;
  public
    constructor Create(const AURL: string; APicture: TPicture);
  end;

implementation

type
  TTMSHackedPicture = class(TPicture);

resourcestring
  SImageSizeIs0 = 'Image size is 0';
  SResourceIsNotOfImageType = 'Resource is not of image type : ';
  SCannotOpenURL = 'Cannot open URL ';
  SCannotOpenHttpConnection = 'Cannot open http connection';
  SCannotOpenInternetSession = 'Cannot open internet session';

{ TDownLoadThread }

constructor TDownLoadThread.Create(const AURL: string; APicture: TPicture);
begin
  inherited Create(False);
  FURL := AURL;
  FPicture := APicture;
  FreeOnTerminate := True;
end;

procedure TDownLoadThread.DownLoad;
{$IFDEF WINDOWS}
const
  cWebImageAgent = 'WebImage';
  cImage = 'IMAGE';
  cBufSize = 4096;
  cHTTPStatusOK = 200;
var
  lBuf: array[0..255] of Char;
  lBufSize: DWORD;
  lCancel: Boolean;
  lErr: Integer;
  lHttp: HINTERNET;
  lHttpSize: Integer;
  lHttpStatus: Integer;
  lIdx: DWORD;
  lLen: DWORD;
  lRB: array[0..cBufSize - 1] of Byte;
  lRBSIZE: DWORD;
  lSession: HINTERNET;
begin
  lSession := InternetOpen(cWebImageAgent,INTERNET_OPEN_TYPE_PRECONFIG,nil,nil,0);
  if (lSession = nil) then
  begin
    DownLoadError(SCannotOpenInternetSession);
    Exit;
  end;

  lHttp := InternetOpenURL(lSession, PChar(FURL), nil, 0,
    INTERNET_FLAG_PRAGMA_NOCACHE or INTERNET_FLAG_NO_CACHE_WRITE or INTERNET_FLAG_RELOAD, 0);

  if (lHttp = nil) then
  begin
    InternetCloseHandle(lSession);
    DownLoadError(SCannotOpenHttpConnection);
    Exit;
  end;

  lBufSize := SizeOf(lBuf);
  lIdx := 0;
  HttpQueryInfo(lHttp, HTTP_QUERY_STATUS_CODE, @lBuf, lBufSize, lIdx);

  Val(lBuf, lHttpStatus, lErr);
  if (lHttpStatus <> cHTTPStatusOK) or (lErr <> 0) then
  begin
    InternetCloseHandle(lSession);
    InternetCloseHandle(lHttp);
    DownLoadError(SCannotOpenURL + FURL);
    Exit;
  end;

  lBufSize := SizeOf(lBuf);
  lIdx := 0;
  HttpQueryInfo(lHttp, HTTP_QUERY_CONTENT_TYPE, @lBuf, lBufSize, lIdx);

  if Pos(cImage, UpperCase(lBuf)) = 0 then
  begin
    InternetCloseHandle(lSession);
    InternetCloseHandle(lHttp);
    DownLoadError(SResourceIsNotOfImageType+FURL);
    Exit;
  end;

  lBufSize := SizeOf(lBuf);
  lIdx := 0;
  HttpQueryInfo(lHttp, HTTP_QUERY_CONTENT_LENGTH, @lBuf, lBufSize, lIdx);

  Val(lBuf, lHttpSize, lErr);

  if ((lHttpSize = 0) or (lErr <> 0)) then
  begin
    InternetCloseHandle(lSession);
    InternetCloseHandle(lHttp);
    DownLoadError(SImageSizeIs0);
    Exit;
  end;

  DownLoadProgress(0,lHttpSize);

  lLen := cBufSize;
  lRBSIZE := cBufSize;

  try
    lCancel := False;
    while (lLen > 0) and (not lCancel) do
    begin
      InternetReadFile(lHttp, @lRB, lRBSIZE, lLen);
      if lLen > 0 then
        FStream.WriteBuffer(lRB, lLen);
      DownLoadProgress(FStream.Size, lHttpSize);
    end;
  finally
    InternetCloseHandle(lHttp);
    InternetCloseHandle(lSession);
  end;
end;
{$ELSE}
begin
  DownloadError(SCannotOpenInternetSession);
  Exit;
end;
{$ENDIF}

procedure TDownLoadThread.DownLoadError(const AError: string);
begin
  //
end;

procedure TDownLoadThread.DownloadProgress(APosition, ASize: Integer);
begin
  //
end;

procedure TDownLoadThread.Execute;
begin
  if FURL = '' then
    Exit;

  FStream := TMemoryStream.Create;

  try
    Download;
    FStream.Position := 0;
    Synchronize(UpdatePicture);
  finally
    FStream.Free;
  end;

end;

procedure TDownLoadThread.UpdatePicture;
begin
  TTMSHackedPicture(FPicture).LoadFromStream(FStream);
end;

end.
