{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Login;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Vcl.Controls, Vcl.Graphics, WEBLib.Controls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Controls, Graphics, WEBLib.Controls;
  {$ENDIF}

type
  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebLoginPanel = class(TCustomControl)
  private
    FPasswordLabel: string;
    FOnLogin: TNotifyEvent;
    FUserLabel: string;
    FPassword: string;
    FUser: string;
    FLoginLabel: string;
    FElementButtonClassname: string;
    FElementID: TElementID;
    FElementInputClassName: string;
    FElementClassName: TElementClassName;
    FBorderColor: TColor;
    FColor: TColor;
    FElementCaptionClassName: string;
    FCaptionLabel: string;
    FElementLabelClassName: string;
    FPadding: integer;
    FMargin: integer;
    FChildOrder: integer;
    procedure SetCaptionLabel(const Value: string);
    procedure SetLoginLabel(const Value: string);
    procedure SetPasswordLabel(const Value: string);
    procedure SetUserLabel(const Value: string);
    procedure SetMargin(const Value: integer);
    procedure SetPadding(const Value: integer);
    procedure SetBorderColor(const Value: TColor);
    procedure SetColor(const Value: TColor);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property BorderColor: TColor read FBorderColor write SetBorderColor default clSilver;
    property CaptionLabel: string read FCaptionLabel write SetCaptionLabel;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color: TColor read FColor write SetColor default clInfoBk;
    property ElementButtonClassName: string read FElementButtonClassname write FElementButtonClassName;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementCaptionClassName: string read FElementCaptionClassName write FElementCaptionClassName;
    property ElementInputClassName: string read FElementInputClassName write FElementInputClassName;
    property ElementLabelClassName: string read FElementLabelClassName write FElementLabelClassName;
    property Font;
    property LoginLabel: string read FLoginLabel write SetLoginLabel;
    property Margin: integer read FMargin write SetMargin default 5;
    property Padding: integer read FPadding write SetPadding default 10;
    property Password: string read FPassword write FPassword;
    property PasswordLabel: string read FPasswordLabel write SetPasswordLabel;
    property User: string read FUser write FUser;
    property UserLabel: string read FUserLabel write SetUserLabel;
    property Visible;
    property OnClick;
    property OnLogin: TNotifyEvent read FOnLogin write FOnLogin;
  end;

implementation

{ TWebLoginPanel }

constructor TWebLoginPanel.Create(AOwner: TComponent);
begin
  inherited;
  Color := clInfoBk;
  BorderColor := clSilver;
  FPasswordLabel := 'Password:';
  FUserLabel := 'User:';
  FCaptionLabel := 'Login';
  FLoginLabel := 'Login';
  FMargin := 5;
  FPadding := 10;
  Width := 185;
  Height := 155;
end;

procedure TWebLoginPanel.Paint;
var
  y: integer;
  th: integer;
begin
  inherited;
  Canvas.Brush.Color := Color;
  Canvas.Pen.Color := BorderColor;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Width := 1;
  Canvas.Rectangle(ClientRect);

  Canvas.Font.Assign(Font);
  th := Canvas.TextHeight('gh');

  y := Padding;

  Canvas.TextOut(Padding, y, FCaptionLabel);

  if FCaptionLabel <> '' then
    y := y + th + 2*Margin;

  Canvas.TextOut(Padding, y, FUserLabel);

  if FUserLabel <> '' then
    y := y + th + 2*Margin;

  Canvas.Brush.Color := clWhite;
  Canvas.Pen.Color := clSilver;
  Canvas.Rectangle(Padding, y, Width - Padding, y + th + Margin);

  y := y + th + 2*Margin;

  Canvas.Brush.Color := Color;

  Canvas.TextOut(Padding, y, FPasswordLabel);

  if FPasswordLabel <> '' then
    y := y + th + 2*Margin;

  Canvas.Brush.Color := clWhite;
  Canvas.Rectangle(Padding, y, Width - Padding, y + th + Margin);

  y := y + th + 2*Margin;

  Canvas.Brush.Color := clSilver;
  Canvas.Pen.Color := clGray;

  Canvas.Rectangle(Width div 2, y, Width - Padding, y + th + Margin);
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut(Width div 2 + 4, y + 1, FLoginLabel);
end;

procedure TWebLoginPanel.SetBorderColor(const Value: TColor);
begin
  FBorderColor := Value;
  Invalidate;
end;

procedure TWebLoginPanel.SetCaptionLabel(const Value: string);
begin
  FCaptionLabel := Value;
  Invalidate;
end;

procedure TWebLoginPanel.SetColor(const Value: TColor);
begin
  FColor := Value;
  Invalidate;
end;

procedure TWebLoginPanel.SetLoginLabel(const Value: string);
begin
  FLoginLabel := Value;
  Invalidate;
end;

procedure TWebLoginPanel.SetMargin(const Value: integer);
begin
  FMargin := Value;
  Invalidate;
end;

procedure TWebLoginPanel.SetPadding(const Value: integer);
begin
  FPadding := Value;
  Invalidate;
end;

procedure TWebLoginPanel.SetPasswordLabel(const Value: string);
begin
  FPasswordLabel := Value;
  Invalidate;
end;

procedure TWebLoginPanel.SetUserLabel(const Value: string);
begin
  FUserLabel := Value;
  Invalidate;
end;

end.
