{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ThreeJsChart;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Types, System.SysUtils, Rtti, Vcl.Controls, Vcl.Forms,
  Winapi.Messages, Winapi.Windows, Vcl.Graphics, WEBLib.Controls,
  Web, libthreejs, js, JSDelphisystem, WEBLib.ThreeJsControls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Types, SysUtils, Controls, Forms, Messages,
  Graphics, Web, libthreejs, js, JSDelphisystem, WEBLib.ThreeJsControls;
  {$ENDIF}

type
  TThreeJsChartLegendPosition = (lpNone, lpTopLeft, lpTopRight, lpBottomLeft, lpBottomRight);
  TThreeJsChartLegendItemShape = (lshSquare, lshCircle);

  TThreeJsCustomChart = class(TThreeJsCustomControl)
  private
    FShowShadows: boolean;
    FAutoRotateAxisLabels: boolean;
    FShowTransparent: boolean;
    FOpacity: double;
    FLegendPosition: TThreeJsChartLegendPosition;
    FLegendStyle: TThreeJsTextStyle;
    FLegendDistance: double;
  public
    procedure CreateChart;
    procedure CreateChartFromData;
    // clears all data and legend
    procedure clearChart;

    // Chart Legend related
    procedure AddChartLegendItem(aName: string; aColor: TColor; aShape: TThreeJsChartLegendItemShape = lshSquare);
    procedure AddChartLegendBox; 
    procedure RemoveChartLegendBox;

    property ShowShadows: boolean read FShowShadows write FShowShadows;
    property AutoRotateAxisLabels: boolean read FAutoRotateAxisLabels write FAutoRotateAxisLabels;
    property ShowTransparent: boolean read FShowTransparent write FShowTransparent;
    property Opacity: double read FOpacity write FOpacity;
    property LegendPosition: TThreeJsChartLegendPosition read FLegendPosition write FLegendPosition;
    property LegendStyle: TThreeJsTextStyle read FLegendStyle write FLegendStyle;
    // To make Legend Text fixed size, this is the fixed distance kept from Camera
    property LegendDistance: double read FLegendDistance write FLegendDistance;
  end;

  TThreeJsChartType = (ctBarChart, ctCylinderChart, ctConeChart, ctLineChart, ctAreaChart);
  TChartsSet = set of TThreeJsChartType;

  TThreeJsChartSeries = class
  private
    FChartType: TThreeJsChartType;

    FValueFormatFloat: string; //global
    // overrides
    FValueAxisFormatFloat: string; //if empty above is used

    // chart item properties
    FItemWidth: double;
    FitemSpace: double;
    FItemTextStyle: TThreeJsTextStyle;
    FPlotWidth: double; //for plot charts only

    // Chart Titles
    FTitle: string;
    FTitleStyle: TThreeJsTextStyle;
    FTitleSpace: double;
    FAxisTitleStyle: TThreeJsTextStyle;
    FCategoryAxisTitle: string;
    FCategoryAxisTitleSpace: double;
    FLegendAxisTitle: string;
    FLegendAxisTitleSpace: double;
    FValueAxisTitle: string;
    FValueAxisTitleSpace: double;

    // Axis properties
    FValueAxisLength: double; //in WebGL Units
    FAutoMarkValueAxis: boolean;
    FValueAxisMarkMaximum, FValueAxisMarkMinimum, FValueAxisMarkStep: double;
    FCategoryAxisLabelStyle: TThreeJsTextStyle;
    FLegendAxisLabelStyle: TThreeJsTextStyle;
    FValueAxisLabelStyle: TThreeJsTextStyle;
    FAxisLabelSpace: double;

  public
    constructor Create(aLegendAxisLabels: TJSArray; aCategoryAxisLabels: TJSArray);
    // Number of data items in the following has to match number of Category Labels
    procedure AddLegendRow(aLegendLabel: string; data: TJSArray);

    function GetValueStr(aValue: JSValue): string;
    procedure SetColors(colors: array of JSValue);
    procedure ResetColors;

    property ChartType: TThreeJsChartType read FChartType write FChartType;
    property ValueFormatFloat: string read FValueFormatFloat write FValueFormatFloat;
    // specify the following only if needed different from above
    property ValueAxisFormatFloat: string read FValueAxisFormatFloat write FValueAxisFormatFloat;

    // chart item properties
    property ItemWidth: double read FItemWidth write FItemWidth;
    property itemSpace: double read FitemSpace write FitemSpace;
    property ItemTextStyle: TThreeJsTextStyle read FItemTextStyle write FItemTextStyle;
    property PlotWidth: double read FPlotWidth write FPlotWidth;

    // titles
    property Title: string read FTitle write FTitle;
    property TitleTextStyle: TThreeJsTextStyle read FTitleStyle write FTitleStyle;
    property TitleSpace: double read FTitleSpace write FTitleSpace;
    property AxisTitleStyle: TThreeJsTextStyle read FAxisTitleStyle write FAxisTitleStyle;
    property CategoryAxisTitle: string read FCategoryAxisTitle write FCategoryAxisTitle;
    property CategoryAxisTitleSpace: double read FCategoryAxisTitleSpace write FCategoryAxisTitleSpace;
    property LegendAxisTitle: string read FLegendAxisTitle write FLegendAxisTitle;
    property LegendAxisTitleSpace: double read FLegendAxisTitleSpace write FLegendAxisTitleSpace;
    property ValueAxisTitle: string read FValueAxisTitle write FValueAxisTitle;
    property ValueAxisTitleSpace: double read FValueAxisTitleSpace write FValueAxisTitleSpace;

    // Axis properties
    property ValueAxisLength: double read FValueAxisLength write FValueAxisLength;
    // Other axes lengths are calculated based on chart items
    property AutoMarkValueAxis: boolean read FAutoMarkValueAxis write FAutoMarkValueAxis;
    // If not Auto, specify the following values
    property ValueAxisMarkMaximum: double read FValueAxisMarkMaximum write FValueAxisMarkMaximum;
    property ValueAxisMarkMinimum: double read FValueAxisMarkMinimum write FValueAxisMarkMinimum;
    property ValueAxisMarkStep: double read FValueAxisMarkStep write FValueAxisMarkStep;

    property CategoryAxisLabelStyle: TThreeJsTextStyle read FCategoryAxisLabelStyle write FCategoryAxisLabelStyle;
    property LegendAxisLabelStyle: TThreeJsTextStyle read FLegendAxisLabelStyle write FLegendAxisLabelStyle;
    property ValueAxisLabelStyle: TThreeJsTextStyle read FValueAxisLabelStyle write FValueAxisLabelStyle;
    property AxisLabelSpace: double read FAxisLabelSpace write FAxisLabelSpace;
  end;

  TThreeJsChartItemEventArgs = class
  private
    FItem: TThreeJsObject3D;
    FLegendLabel: string;
    FCategoryLabel: string;
    FValue: double;
    FValueStr: string;
    FXIndex: Integer;
    FZIndex: Integer;
  public
    property Item: TThreeJsObject3D read FItem write FItem;
    property LegendLabel: string read FLegendLabel write FLegendLabel;
    property CategoryLabel: string read FCategoryLabel write FCategoryLabel;
    property Value: double read FValue write FValue;
    property ValueStr: string read FValueStr write FValueStr;
    property XIndex: Integer read FXIndex write FXIndex;
    property ZIndex: Integer read FZIndex write FZIndex;
  end;

  //Just sending the object is not enough becauses for plot
  //objects, the xIndex and the itemValue has to be determined
  //based on the click point.
  TThreeJsItemEvent = procedure(Sender: TObject; Args: TThreeJsChartItemEventArgs) of object;
  TThreeJsItemNotifyEvent = procedure(Sender: TObject; chartItem: TThreeJsObject3D) of object;


{$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebThreeJsChart = class(TThreeJsCustomChart)
  private
    FSeries: TThreeJsChartSeries;
    FShowValuesOnItems: boolean;
    FSizeZAxis, FSizeXAxis: double;

    FOnItemClick: TThreeJsItemEvent;
    FOnItemExit: TThreeJsItemNotifyEvent;
    FOnItemDblClick: TThreeJsItemNotifyEvent;
    FOnItemMouseMove: TThreeJsItemEvent;
    FOnItemMouseEnter: TThreeJsItemNotifyEvent;
    FOnItemMouseLeave: TThreeJsItemNotifyEvent;
  protected
    function GetGlowChartItem(chartItem: TThreeJsObject3D): boolean;
    procedure SetGlowChartItem(chartItem: TThreeJsObject3D; avalue: boolean);
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateChartFromData;
    procedure clearItemLabels;

    function isPlotTypeChart: boolean;

    // For plot type objects, xIndex is ignored
    function GetItemObject(xIndex, zIndex: integer): TThreeJsObject3D;
    function ShowChartValue(xIndex, zIndex: integer; aValueStr: string): TThreeJsObject3D;
    procedure RemoveChartValue(aChartItem: TThreeJsObject3D);

    property Series: TThreeJsChartSeries read FSeries write FSeries;

    property Glow[chartItem: TThreeJsObject3D]: boolean read getGlowChartItem write setGlowChartItem;
    property SizeXAxis: double read FSizeXAxis write FSizeXAxis;
    property SizeZAxis: double read FSizeZAxis write FSizeZAxis;

  published
    property ShowShadows;
    property AutoRotateAxisLabels;
    property ShowTransparent;
    property Opacity;
    property LegendPosition;
    property ShowValuesOnItems: boolean read FShowValuesOnItems write FShowValuesOnItems;
    property OnClick;
    property OnDblClick;
    property OnItemClick: TThreeJsItemEvent read FOnItemClick write FOnItemClick;
    property OnItemExit: TThreeJsItemNotifyEvent read FOnItemExit write FOnItemExit;
    property OnItemDblClick: TThreeJsItemNotifyEvent read FOnItemDblClick write FOnItemDblClick;
    property OnItemMouseMove: TThreeJsItemEvent read FOnItemMouseMove write FOnItemMouseMove;
    property OnItemMouseEnter: TThreeJsItemNotifyEvent read FOnItemMouseEnter write FOnItemMouseEnter;
    property OnItemMouseLeave: TThreeJsItemNotifyEvent read FOnItemMouseLeave write FOnItemMouseLeave;
  end;

implementation

procedure TThreeJsCustomChart.CreateChart;
begin
end;
procedure TThreeJsCustomChart.clearChart;
begin
end;

procedure TThreeJsCustomChart.CreateChartFromData;
begin
end;


procedure TThreeJsCustomChart.AddChartLegendItem(aName: string; aColor: TColor; aShape: TThreeJsChartLegendItemShape = lshSquare);
begin
end;

procedure TThreeJsCustomChart.AddChartLegendBox; 
begin
end;

procedure TThreeJsCustomChart.RemoveChartLegendBox;
begin
end;

constructor TThreeJsChartSeries.create(aLegendAxisLabels: TJSArray; aCategoryAxisLabels: TJSArray);
begin
end;

procedure TThreeJsChartSeries.addLegendRow(aLegendLabel: string; data: TJSArray);
begin
end;

function TThreeJsChartSeries.getValueStr(aValue: JSValue): string;
begin
  result := '';
end;

procedure TThreeJsChartSeries.setColors(colors: array of JSValue);
begin
end;

procedure TThreeJsChartSeries.resetColors;
begin
end;

function TWebThreeJsChart.getGlowChartItem(chartItem: TThreeJsObject3D): boolean;
begin
  result := false;
end;

procedure TWebThreeJsChart.setGlowChartItem(chartItem: TThreeJsObject3D; avalue: boolean);
begin
end;

constructor TWebThreeJsChart.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ShowShadows := false;
  AutoRotateAxisLabels := false;
  ShowTransparent := false;
  Opacity := 0.6;
  LegendPosition := lpTopRight;
  ShowValuesOnItems := false;
end;

procedure TWebThreeJsChart.CreateChartFromData;
begin
end;

procedure TWebThreeJsChart.clearItemLabels;
begin
end;

function TWebThreeJsChart.isPlotTypeChart: boolean;
begin
  result := false;
end;

function TWebThreeJsChart.getItemObject(xIndex, zIndex: integer): TThreeJsObject3D;
begin
  result := nil;
end;

function TWebThreeJsChart.showChartValue(xIndex, zIndex: integer; aValueStr: string): TThreeJsObject3D;
begin
  result := nil;
end;

procedure TWebThreeJsChart.removeChartValue(aChartItem: TThreeJsObject3D);
begin
end;

end.
