{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}
unit libindexeddb;

interface

uses jsdelphisystem, Types, JS, Web;

type
  TMIDBTransactionMode = class;
  TMIDBTransaction = class;
  TMIDBKeyRange = class;
  TMIDBIndex = class;
  TMIDBCursorDirection = class;
  TMIDBCursor = class;
  TMIDBObjectStore = class;
  TMIDBRequest = class;
  TMIDBOpenDBRequest = class;
  TMIDBDatabase = class;
  TMIDBFactory = class;

  TMIDBTransactionMode = class
    const
      readonly = 'readonly';
      readwrite = 'readwrite';
      versionchange = 'versionchange';
  end;

  TMIDBTransaction = class(TJSEventTarget)
  private
    FDB: TMIDBDatabase;
    FError: JSValue;
    FMode: string;
    FObjectStoreNames: TStringDynArray;
  public
    procedure abort;
    function objectStore(aName: string): TMIDBObjectStore;
    property db: TMIDBDatabase read FDB;
    property mode: string read FMode;
    property objectStoreNames: TStringDynArray read FObjectStoreNames;
    property error: JSValue read FError;
  end;

  TMIDBKeyRange = class
  private
    FLower: JSValue;
    FLowerOpen: Boolean;
    FUpper: JSValue;
    FUpperOpen: Boolean;
  public
    class function bound(aLower: JSValue; aUpper: JSValue): TMIDBKeyRange; overload;
    class function bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean): TMIDBKeyRange; overload;
    class function bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean; aUpperOpen: Boolean): TMIDBKeyRange; overload;
    class function lowerBound(aLower: JSValue): TMIDBKeyRange; overload;
    class function lowerBound(aLower: JSValue; aOpen: Boolean): TMIDBKeyRange; overload;
    class function only(aValue: JSValue): TMIDBKeyRange;
    class function upperBound(aUpper: JSValue): TMIDBKeyRange; overload;
    class function upperBound(aUpper: JSValue; aOpen: Boolean): TMIDBKeyRange; overload;
    function includes(aValue: JSValue): Boolean;
    property lower: JSValue read FLower;
    property lowerOpen: Boolean read FLowerOpen;
    property upper: JSValue read FUpper;
    property upperOpen: Boolean read FUpperOpen;
  end;
  TMIDBIndexParameters = record
    unique : Boolean;
    multiEntry : Boolean;
    locale : string;
  end;

  TMIDBIndex = class
  private
    FKeyPath: JSValue;
    FMultiEntry: Boolean;
    FObjectStore: TMIDBObjectStore;
    FUnique: Boolean;
  public
    name: string;
    function count: TMIDBRequest;
    function get(aKey: jsValue): TMIDBRequest; overload;
    function get(aKey: TMIDBKeyRange): TMIDBRequest; overload;
    function getAll : TMIDBRequest; overload;
    function getAll(aKey: jsValue): TMIDBRequest; overload;
    function getAll(aKey: TMIDBKeyRange): TMIDBRequest; overload;
    function getAll(aKey: jsValue; ACount: NativeInt): TMIDBRequest; overload;
    function getAll(aKey: TMIDBKeyRange; ACount: NativeInt): TMIDBRequest; overload;
    function getAllKeys(aKey: jsValue): TMIDBRequest; overload;
    function getAllKeys(aKey: TMIDBKeyRange): TMIDBRequest; overload;
    function getAllKeys(aKey: jsValue; ACount: NativeInt): TMIDBRequest; overload;
    function getAllKeys(aKey: TMIDBKeyRange; ACount: NativeInt): TMIDBRequest; overload;
    function getKey(aKey: jsValue): TMIDBRequest;
    function openCursor: TMIDBRequest; overload;
    function openCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function openCursor(aKeyRange: TMIDBKeyRange; ADirection: string): TMIDBRequest; overload;
    function openKeyCursor: TMIDBRequest; overload;
    function openKeyCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function openKeyCursor(aKeyRange: TMIDBKeyRange; ADirection: string): TMIDBRequest; overload;
    property keyPath: JSValue read FKeyPath;
    property multiEntry: Boolean read FMultiEntry;
    property objectStore: TMIDBObjectStore read FObjectStore;
    property unique: Boolean read FUnique;
  end;

  TMIDBCursorDirection = class
    const
      next = 'next';
      nextUnique = 'nextUnique';
      prev = 'prev';
      prevUnique = 'prevUnique';
  end;

  TMIDBCursor = class
  private
    FDirection: string;
    FKey: JSValue;
    FValue : JSValue; 
    FPrimaryKey: JSValue;
    FSource: JSValue;
    FSourceAsIndex: TMIDBIndex;
    FSourceAsStore: TMIDBObjectStore;
  public
    procedure advance(aCount: NativeInt); overload;
    procedure advance(aKey: JSValue); overload;
    procedure continue(aKey: JSValue); overload;
    procedure continue; overload;
    procedure continuePrimaryKey(aKey: JSValue); overload;
    procedure continuePrimaryKey(aKey,aPrimaryKey : JSValue); overload;
    procedure delete;
    procedure update(aValue: JSValue);
    property source: JSValue read FSource;
    property sourceAsStore: TMIDBObjectStore read FSourceAsStore;
    property sourceAsIndex: TMIDBIndex read FSourceAsIndex;
    property key: JSValue read FKey;
    Property Value : JSValue Read FValue;
    property primaryKey: JSValue read FPrimaryKey;
    property direction: string read FDirection;
  end;

  TMIDBObjectStore = class(TJSEventTarget)
  private
    FAutoIncrement : boolean; 
  public
    function add(aValue: JSValue; aKey: string): TMIDBRequest; overload;
    function add(aValue: JSValue): TMIDBRequest; overload;
    function clear: TMIDBRequest;
    function delete(aKey: string): TMIDBRequest; overload;
    function delete(aKey : JSValue) : TMIDBRequest; overload;
    function delete(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function get(aKey: string): TMIDBRequest; overload;
    function get(aKey : JSValue) : TMIDBRequest; overload;
    function get(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function getKey(aKey: string): TMIDBRequest; overload;
    function getKey(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function getAll: TMIDBRequest; overload;
    function getAll(aKey: string): TMIDBRequest; overload;
    function getAll(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function getAll(aKey: string; aCount: NativeInt): TMIDBRequest; overload;
    function getAll(aKeyRange: TMIDBKeyRange; aCount: NativeInt): TMIDBRequest; overload;
    function getAllKeys(aKey: string): TMIDBRequest; overload;
    function getAllKeys(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function getAllKeys(aKey: string; aCount: NativeInt): TMIDBRequest; overload;
    function getAllKeys(aKeyRange: TMIDBKeyRange; aCount: NativeInt): TMIDBRequest; overload;
    function createIndex(aIndexName: string; KeyPath: string): TMIDBIndex; overload;
    function createIndex(aIndexName: string; KeyPath: string; Options: TMIDBIndexParameters): TMIDBIndex; overload;
    function createIndex(aIndexName: string; KeyPath: Array of string): TMIDBIndex; overload;
    function createIndex(aIndexName: string; KeyPath: Array of string; Options: TMIDBIndexParameters): TMIDBIndex; overload;
    function createIndex (aIndexName : String; KeyPath : JSValue; Options : TMIDBIndexParameters)  : TMIDBIndex; overload;
    procedure deleteIndex(aIndexName: string);
    function index(aIndexName: string): TMIDBIndex;
    function put(aValue: JSValue; aKey: string): TMIDBRequest; overload;
    function put(aValue : JSValue; aKey : JSValue) : TMIDBRequest; overload;
    function put(aValue: JSValue): TMIDBRequest; overload;
    function openCursor: TMIDBRequest; overload;
    function openCursor(aKey: string): TMIDBRequest; overload;
    function openCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function openCursor(aKey: string; aDirection: string): TMIDBRequest; overload;
    function openCursor(aKeyRange: TMIDBKeyRange; aDirection: string): TMIDBRequest; overload;
    function openKeyCursor: TMIDBRequest; overload;
    function openKeyCursor(aKey: string): TMIDBRequest; overload;
    function openKeyCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    function openKeyCursor(aKey: string; aDirection: string): TMIDBRequest; overload;
    function openKeyCursor(aKeyRange: TMIDBKeyRange; aDirection: string): TMIDBRequest; overload;
    function count: TMIDBRequest; overload;
    function count(aKey: string): TMIDBRequest; overload;
    function count(aKeyRange: TMIDBKeyRange): TMIDBRequest; overload;
    property Indexes[aIndexName : string]: TMIDBIndex read index;
    Property autoIncrement : boolean read FAutoIncrement;
  end;

  TMIDBRequest = class(TJSEventTarget)
  private
    Ferror: JSValue;
    FReadyState: string;
    FResult: JSValue;
    FResultDatabase: TMIDBDatabase;
    FResultIndex: TMIDBIndex;
    FResultObjectStore: TMIDBObjectStore;
    FResultCursor: TMIDBCursor;
    FSourceDatabase: TMIDBDatabase;
    FSourceIndex: TMIDBIndex;
    FSourceObjectStore: TMIDBObjectStore;
    FSourceCursor: TMIDBCursor;
    FSource: JSValue;
    FTransaction: TMIDBTransaction;
  public
    onerror: TJSEventHandler;
    onsuccess: TJSEventHandler;
    property error: JSValue read FError;
    property readyState: string read FReadyState;
    property result: JSValue read FResult;
    property resultAsObjectStore: TMIDBObjectStore read FResultObjectStore;
    property resultAsCursor: TMIDBCursor read FResultCursor;
    property resultAsIndex: TMIDBIndex read FResultIndex;
    property resultAsDatabase: TMIDBDatabase read FResultDatabase;
    property source: JSValue read FSource;
    property sourceAsObjectStore: TMIDBObjectStore read FSourceObjectStore;
    property sourceAsCursor: TMIDBCursor read FSourceCursor;
    property sourceAsIndex: TMIDBIndex read FSourceIndex;
    property sourceAsDatabase: TMIDBDatabase read FSourceDatabase;
    property transaction: TMIDBTransaction read FTransaction;
  end;

  TMIDBOpenDBRequest = class(TMIDBRequest)
  public
    onblocked: TJSEventHandler;
    onupgradeneeded: TJSEventHandler;
  end;
  TJSCreateObjectStoreOptions = record
    keyPath : jsValue;
    autoIncrement : Boolean;
  end;

  TMIDBDatabase = class(TJSEventTarget)
  private
    FName: string;
    FobjectStoreNames: TStringDynArray;
    FVersion: Integer;
    FObjectStore: TMIDBObjectStore;
  public
    onversionchange : TJSEventHandler; 
    procedure close;
    function createObjectStore(aName: string): TMIDBObjectStore; overload;
    function createObjectStore(aName: string; Options: TJSCreateObjectStoreOptions): TMIDBObjectStore; overload;
    procedure deleteObjectStore(aName: string);
    function transaction(aStoreNames: Array of string): TMIDBTransaction; overload;
    function transaction(aStoreNames: Array of string; aMode: string): TMIDBTransaction; overload;
    property name: string read FName;
    property version: Integer read FVersion;
    property objectStoreNames: TStringDynArray read FobjectStoreNames;
    property objectStore: TMIDBObjectStore read FObjectStore;
  end;

  TMIDBFactory = class(TJSEventTarget)
  public
    function open(aName: string): TMIDBOpenDBRequest; overload;
    function open(aName: string; aVersion: Integer): TMIDBOpenDBRequest; overload;
    function deleteDatabase(aName: string): TMIDBOpenDBRequest;
    function cmp(a: jsValue; b: jsValue): NativeInt;
  end;

  TIDBError = Class(TJSObject)
  private
    FMessage: String;
    FName: String;
  Public
    property message : String Read FMessage;
    property name : String Read FName;
  end;

implementation
{$HINTS OFF}
{$WARNINGS OFF}

procedure TMIDBTransaction.abort;

begin
end;

function TMIDBTransaction.objectStore(aName: string): TMIDBObjectStore;

begin
  result := nil;
end;

class function TMIDBKeyRange.bound(aLower: JSValue; aUpper: JSValue): TMIDBKeyRange;

begin
  result := nil;
end;

class function TMIDBKeyRange.bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean): TMIDBKeyRange;

begin
  result := nil;
end;

class function TMIDBKeyRange.bound(aLower: JSValue; aUpper: JSValue; aLowerOpen: Boolean; aUpperOpen: Boolean): TMIDBKeyRange;

begin
  result := nil;
end;

class function TMIDBKeyRange.lowerBound(aLower: JSValue): TMIDBKeyRange;

begin
  result := nil;
end;

class function TMIDBKeyRange.lowerBound(aLower: JSValue; aOpen: Boolean): TMIDBKeyRange;

begin
  result := nil;
end;

class function TMIDBKeyRange.only(aValue: JSValue): TMIDBKeyRange;

begin
  result := nil;
end;

class function TMIDBKeyRange.upperBound(aUpper: JSValue): TMIDBKeyRange;

begin
  result := nil;
end;

class function TMIDBKeyRange.upperBound(aUpper: JSValue; aOpen: Boolean): TMIDBKeyRange;

begin
  result := nil;
end;

function TMIDBKeyRange.includes(aValue: JSValue): Boolean;

begin
end;

function TMIDBIndex.count: TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.get(aKey: jsValue): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.get(aKey: TMIDBKeyRange): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAll : TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAll(aKey: jsValue): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAll(aKey: TMIDBKeyRange): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAll(aKey: jsValue; ACount: NativeInt): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAll(aKey: TMIDBKeyRange; ACount: NativeInt): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAllKeys(aKey: jsValue): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAllKeys(aKey: TMIDBKeyRange): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAllKeys(aKey: jsValue; ACount: NativeInt): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getAllKeys(aKey: TMIDBKeyRange; ACount: NativeInt): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.getKey(aKey: jsValue): TMIDBRequest;

begin
  result := nil;
end;

function TMIDBIndex.openCursor: TMIDBRequest;

begin
end;

function TMIDBIndex.openCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBIndex.openCursor(aKeyRange: TMIDBKeyRange; ADirection: string): TMIDBRequest;

begin
end;

function TMIDBIndex.openKeyCursor: TMIDBRequest;

begin
end;

function TMIDBIndex.openKeyCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBIndex.openKeyCursor(aKeyRange: TMIDBKeyRange; ADirection: string): TMIDBRequest;

begin
end;

procedure TMIDBCursor.advance(aCount: NativeInt);

begin
end;

procedure TMIDBCursor.advance(aKey: JSValue);

begin
end;

procedure TMIDBCursor.continue(aKey: JSValue);

begin
end;

procedure TMIDBCursor.continue;

begin
end;

procedure TMIDBCursor.continuePrimaryKey(aKey: JSValue);

begin
end;

procedure TMIDBCursor.continuePrimaryKey(aKey: JSValue; aPrimaryKey: JSValue);

begin
end;

procedure TMIDBCursor.delete;

begin
end;

procedure TMIDBCursor.update(aValue: JSValue);

begin
end;

function TMIDBObjectStore.add(aValue: JSValue; aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.add(aValue: JSValue): TMIDBRequest;

begin
end;

function TMIDBObjectStore.clear: TMIDBRequest;

begin
end;

function TMIDBObjectStore.delete(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.delete(aKey : JSValue) : TMIDBRequest;

begin
end;

function TMIDBObjectStore.delete(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBObjectStore.get(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.get(aKey : JSValue) : TMIDBRequest;

begin
end;

function TMIDBObjectStore.get(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getKey(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getKey(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAll: TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAll(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAll(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAll(aKey: string; aCount: NativeInt): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAll(aKeyRange: TMIDBKeyRange; aCount: NativeInt): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAllKeys(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAllKeys(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAllKeys(aKey: string; aCount: NativeInt): TMIDBRequest;

begin
end;

function TMIDBObjectStore.getAllKeys(aKeyRange: TMIDBKeyRange; aCount: NativeInt): TMIDBRequest;

begin
end;

function TMIDBObjectStore.createIndex(aIndexName: string; KeyPath: string): TMIDBIndex;

begin
end;

function TMIDBObjectStore.createIndex(aIndexName: string; KeyPath: string; Options: TMIDBIndexParameters): TMIDBIndex;

begin
end;

function TMIDBObjectStore.createIndex(aIndexName: string; KeyPath: Array of string): TMIDBIndex;

begin
end;

function TMIDBObjectStore.createIndex(aIndexName: string; KeyPath: Array of string; Options: TMIDBIndexParameters): TMIDBIndex;

begin
end;

function TMIDBObjectStore.createIndex(aIndexName: string; KeyPath: JSValue; Options: TMIDBIndexParameters): TMIDBIndex;

begin
end;

procedure TMIDBObjectStore.deleteIndex(aIndexName: string);

begin
end;

function TMIDBObjectStore.index(aIndexName: string): TMIDBIndex;

begin
end;

function TMIDBObjectStore.put(aValue: JSValue; aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.put(aValue : JSValue; aKey : JSValue) : TMIDBRequest;

begin
end;

function TMIDBObjectStore.put(aValue: JSValue): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openCursor: TMIDBRequest;

begin
end;

function TMIDBObjectStore.openCursor(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openCursor(aKey: string; aDirection: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openCursor(aKeyRange: TMIDBKeyRange; aDirection: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openKeyCursor: TMIDBRequest;

begin
end;

function TMIDBObjectStore.openKeyCursor(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openKeyCursor(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openKeyCursor(aKey: string; aDirection: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.openKeyCursor(aKeyRange: TMIDBKeyRange; aDirection: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.count: TMIDBRequest;

begin
end;

function TMIDBObjectStore.count(aKey: string): TMIDBRequest;

begin
end;

function TMIDBObjectStore.count(aKeyRange: TMIDBKeyRange): TMIDBRequest;

begin
end;

procedure TMIDBDatabase.close;

begin
end;

function TMIDBDatabase.createObjectStore(aName: string): TMIDBObjectStore;

begin
end;

function TMIDBDatabase.createObjectStore(aName: string; Options: TJSCreateObjectStoreOptions): TMIDBObjectStore;

begin
end;

procedure TMIDBDatabase.deleteObjectStore(aName: string);

begin
end;

function TMIDBDatabase.transaction(aStoreNames: Array of string): TMIDBTransaction;

begin
end;

function TMIDBDatabase.transaction(aStoreNames: Array of string; aMode: string): TMIDBTransaction;

begin
end;

function TMIDBFactory.open(aName: string): TMIDBOpenDBRequest;

begin
end;

function TMIDBFactory.open(aName: string; aVersion: Integer): TMIDBOpenDBRequest;

begin
end;

function TMIDBFactory.deleteDatabase(aName: string): TMIDBOpenDBRequest;

begin
end;

function TMIDBFactory.cmp(a: jsValue; b: jsValue): NativeInt;

begin
end;


// end of dummy implementations
end.
