{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.RadServerCDS;

//{$define cdsdebug}

interface

uses
   Classes, JS, Web, SysUtils, WEBLib.CDS, JSONDataSet,
   DB, WebLib.REST, strutils;

type
  TRadServerClientDataset = class;
  TRadServerClientDataProxy = class;

  TRadServer = class(TRESTClient)
  protected
    procedure HttpsCommand(Command, URL, Data, ContentType: string; headers: TCoreCloudHeaders); override;
    procedure DoConnect(Username, Password: String; IsSignup: Boolean);
    procedure HandleLogin(Sender: TObject; AResponse: String;
                               var Handled: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property OnConnect: TNotifyEvent read FOnConnect write FOnConnect;
  end;

  TRadServerEntityUpdate = class(TCollectionItem)
  protected
    procedure HandleInsert(Sender: TObject; AResponse: String; var Handled: Boolean);
    procedure HandleUpdateDelete(Sender: TObject; AResponse: String; var Handled: Boolean);
  public
    constructor Create(Collection: TCollection);  override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function Update: Boolean;
    function Insert: Boolean;
    function Delete: Boolean;
    property CustomData: JSValue read FCustomData write FCustomData;
    property Data: TJSObject read FData write FData;
  end;

  TRadServerEntityUpdates = class(TOwnedCollection)
  protected
    property DataProxy: TRadServerClientDataProxy read FDataProxy;
  public
    constructor Create(AOwner: TPersistent); reintroduce;
    function Add: TRadServerEntityUpdate; reintroduce;
    function Insert(Index: Integer): TRadServerEntityUpdate; reintroduce;
    property Items[Index: Integer]: TRadServerEntityUpdate read GetItems write SetItems; default;
  end;

  TRadServerClientDataProxy = Class(TDataProxy)
 protected
    function GetDataRequest(aOptions: TLoadOptions; aAfterRequest: TDataRequestEvent; aAfterLoad: TDatasetLoadEvent) : TDataRequest; override;
    function DoGetData(aRequest : TDataRequest) : Boolean; override;
    procedure DoOnError(errMsg: String);
    procedure DoOnEntityError(anEntityUpdate: TRadServerEntityUpdate; errMsg: String);
    procedure DoOnEntityUpdateDelete(anEntityUpdate: TRadServerEntityUpdate);
    procedure DoOnEntityInsert(anEntityUpdate: TRadServerEntityUpdate);
    procedure DoOnConnect(Sender: TObject);
    procedure GetEntities;
    procedure HandleGetEntities(Sender: TObject; AResponse: String; var Handled: Boolean);
    procedure ProcessUpdate(desc: TRecordUpdateDescriptor);
    procedure Close;
    // Dummy but necessary only as it is an Abstract method
    function ProcessUpdateBatch(aBatch: TRecordUpdateBatch): Boolean; override;
    function GetKeyFieldName: String;
    function GetTableName: String;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property TableName: String read GetTableName;
    property Db: TRadServer read FDb;
    property KeyFieldName: String read GetKeyFieldName;
  end;

  TRadServerErrorEvent = procedure(Sender: TObject; errMsg: String) of object;
  TRadServerGetQueryParameterEvent = function(Sender: TObject; aCommand: THTTPCommand; var Handled: Boolean): String of object;
  TRadServerSignupUserExistsEvent = procedure(Sender: TObject; UserName: String) of object;

  TRadServerClientDataset = Class(TClientDataSet)
  protected
    procedure InternalClose; override;
    Function AddToChangeList(aChange : TUpdateStatus) : TRecordUpdateDescriptor ; override;
    procedure SetActive(Value: Boolean); override;
    procedure AfterLoadDFMValues; override;
    procedure DoAfterOpen; override;
    procedure DoOnError(errMsg: String);
    procedure UpdateStarts;
    procedure UpdateEnds;
    function GetQueryParameter(aCommand: THTTPCommand): String; virtual;
    procedure FixKeyFieldCase;
    function IsKeyFieldValid: Boolean;
    function DoOnGetQueryParameter(aCommand: THTTPCommand; var Handled: Boolean): String;
    property DataLoaded: Boolean read FDataLoaded write FDataLoaded;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh(Force: Boolean=False); reintroduce;
    procedure AddSortFieldDef(aField: String; isAscending: Boolean);
    procedure ClearSortFieldDefs;
    procedure Login(aUserName, aPassword: String; IsSignUp: Boolean=False);

  published
    property RadServerURL: String read FRadServerURL write FRadServerURL;
    property TableName: String read FTableName write FTableName;
    property KeyFieldName: String read FKeyFieldName write FKeyFieldName;

    // If an AppSecret has been set up in Rad Server, it must
    // be specified in addition to the login credentials
    property AppSecret: String read FAppSecret write FAppSecret;

    // If a MasterSecret has been set up in Rad Server, specify
    // this as an Altnernative to all of above, including Login.
    // It will always get access for all operations.
    property MasterSecret: String read FMasterSecret write FMasterSecret;

    property OnError: TRadServerErrorEvent read FOnError write FOnError;
    property OnUserExists: TRadServerSignupUserExistsEvent read FOnUserExists write FOnUserExists;

    // App can send custom query parameters to attach to a REST command by
    // this event. Note that if it is used for httpGET then the App should
    // take care of sorting parameters completely. In its absence, the component
    // inserts sorting parameters with the assumption of using FireDac
    property OnGetQueryParameter: TRadServerGetQueryParameterEvent read FOnGetQueryParameter write FOnGetQueryParameter;
  end;

  TWebRadServerClientDataset = class(TRadServerClientDataset);

implementation
end.
