{$Error This is a closed source unit. You cannot compile it, it was already compiled.}
{$Error Probably the IDE has cleaned up and you have to unpack the zip again.}
{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.SyntaxMemo;

interface

uses
  Classes, WEBLib.Controls, Web, JS, libace, SysUtils, Types;

type
  TSyntaxMemoMode = string;
  TSyntaxMemoTheme = string;

  TSyntaxMemoChangeCursorEvent = procedure(Sender: TObject; ARow: Integer; AColumn: Integer) of object;
  TSyntaxMemoChangeSelectionEvent = procedure(Sender: TObject; ASelectedText: string) of object;

  TSyntaxAutocomplete = (saNone, saBasic, saLive);
  TSyntaxWrapping = (swNone, swPrintMargin, swValue, swView);
  TSyntaxTextDirection = (stdLeftToRight, stdRightToLeft);

  TSyntaxCompletion = class(TCollectionItem)
  public
    constructor Create(ACollection: TCollection); override;
  published
    property Caption: string read FCaption write FCaption;
    property Meta: string read FMeta write FMeta;
    property Name: string read FName write FName;
    property Score: Integer read FScore write FScore default 0;
    property Value: string read FValue write FValue;
  end;

  TSyntaxCompleter = class(TCollection)
  protected
    procedure Changed; virtual; reintroduce;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create;
    function Add: TSyntaxCompletion; reintroduce;
    function Insert(Index: Integer): TSyntaxCompletion; reintroduce;
    property Items[Index: integer]: TSyntaxCompletion read GetItem write SetItem;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TSyntaxMemo = class(TCustomControl)
  protected
    procedure AutocompleteChanged(Sender: TObject);
    procedure BindEvents; override;
    procedure DoChange; virtual;
    procedure DoChangeCursorPosition(ARow: Integer; AColumn: Integer);
    procedure DoChangeSelection(ASelectedText: string);
    procedure DoLinesChange(Sender: TObject);
    procedure DragDrop(Source: TObject; X, Y: Integer); reintroduce;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean); reintroduce;
    procedure EditorResize(Sender: TObject);
    procedure RemoveCompleter(ACompleter: TAceCompleter);
    procedure UnbindEvents; override;
    procedure UpdateElement; override;
    function CreateElement: TJSElement; override;
    function GetElementBindHandle: TJSEventTarget; override;
    function GetInstanceName: string;
    function HandleDoChangeCursor(Event: TJSMouseEvent): Boolean; virtual;
    function HandleDoChangeSelection(Event: TJSMouseEvent): Boolean; virtual;
    function HandleDoDragOver(aEvent: TJSDragEvent): boolean; virtual;
    function HandleDoDrop(aEvent: TJSDragEvent): Boolean; virtual;
    procedure Loaded; override;
  public
    procedure CreateInitialize; override;
    destructor Destroy; override;
    procedure BeginUpdate; override;
    procedure Clear;
    procedure DisableLocalKeywords;
    procedure EndUpdate; override;
    procedure Find(AText: string);
    procedure FindAll(AText: string);
    procedure FindNext;
    procedure FindPrevious;
    procedure Focus;
    procedure InitializeKeyWords(ACompleter: TSyntaxCompleter);
    procedure InsertText(AText: string); overload;
    procedure InsertText(APosition: TPoint; AText: string); overload;
    procedure PreloadPascalKeywords;
    procedure Redo;
    procedure RemoveSelectedText;
    procedure RemoveCustomAutocompleter;
    procedure RemovePascalKeywords;
    procedure Replace(AReplacement: string); overload; //replace the selected text
    procedure Replace(AText, AReplacement: string); overload;
    procedure ReplaceAll(AReplacement: string); overload; //replace the selected text
    procedure ReplaceAll(AText, AReplacement: string); overload;
    procedure SelectAll;
    procedure Undo;
    procedure Unselect;
    function OpenSearchBox: Boolean;
    property CaretPosition: TPoint read GetCaretPosition write SetCaretPosition;
    property FontName: string read GetFont write SetFontName;
    property Text: string read GetText write SetText;
  published
    property Align;
    property Anchors;
    property AlignWithMargins;
    property Autocompletion: TSyntaxAutocomplete read FAutocompletion write SetAutocompletion;
    property CustomAutocomplete: TSyntaxCompleter read FCustomAutocomplete write SetCustomAutocomplete;
    property FadeFoldWidgets: Boolean read GetFadeFoldWidgets write SetFadeFoldWidgets;
    property FixedGutterWidth: Boolean read FFixedGutterWidth write SetFixedGutterWidth;
    property FontSize: Integer read GetFontSize write SetFontSize;
    property Height;
    property HighlightActiveLine: Boolean read GetHighlightActiveLine write SetHighlightActiveLine;
    property Left;
    property Lines: TStringList read GetLines write SetLines;
    property Mode: TSyntaxMemoMode read FMode write SetMode;
    property PersistentHorizontalScrollbar: Boolean read GetPersistentHorizontalScrollbar write SetPersistentHorizontalScrollbar;
    property PersistentVerticalScrollbar: Boolean read GetPersistentVerticalScrollbar write SetPersistentVerticalScrollbar;
    property PrintMargin: Integer read GetPrintMargin write SetPrintMargin default 80;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly;
    property SelLength: Integer read GetSelLength write SetSelLength;
    property SelStart: Integer read GetSelStart write SetSelStart;
    property ShowFoldWidgets: Boolean read GetShowFoldWidgets write SetShowFoldWidgets;
    property ShowGutter: Boolean read GetShowGutter write SetShowGutter;
    property ShowIndentGuides: Boolean read GetShowIndentGuides write SetShowIndentGuides;
    property ShowInvisibles: Boolean read GetShowInvisibles write SetShowInvisibles;
    property ShowLineNumbers: Boolean read FShowLineNumbers write SetShowLineNumbers;
    property ShowPrintMargin: Boolean read GetShowPrintMargin write SetShowPrintMargin;
    property SoftTabs: Boolean read FSoftTabs write SetSoftTabs default True;
    property TextDirection: TSyntaxTextDirection read FTextDirection write SetTextDirection;
    property Theme: string read FTheme write SetTheme;
    property TabOrder;
    property TabSize: Integer read GetTabSize write SetTabSize;
    property TabStop;
    property Top;
    property Visible;
    property Width;
    property WordWrap: TSyntaxWrapping read FWordWrap write SetWordWrap;
    property WordWrapIndented: Boolean read FWordWrapIndent write SetWordWrapIndent;
    property WordWrapValue: Integer read FWordWrapValue write SetWordWrapValue;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnChangeCursor: TSyntaxMemoChangeCursorEvent read FOnChangeCursor write FOnChangeCursor;
    property OnChangeSelection: TSyntaxMemoChangeSelectionEvent read FOnChangeSelection write FOnChangeSelection;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TWebSyntaxMemo = class(TSyntaxMemo);

implementation
end.
