{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit TMSWebCoreDsgnRegister;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  // LCL
  Forms, Controls, dialogs, strutils,
  // LazUtils
  LazLoggerBase, LazFileUtils, LazVersion,
  // IdeIntf
  ProjectIntf, CompOptsIntf, LazIDEIntf, IDEOptionsIntf, MacroIntf,
  FormEditingIntf, IDEExternToolIntf, IDEOptEditorIntf, PackageIntf,
  PackageLinkIntf, IDEDialogs, PropEdits,
  // TMS Web Core
  TMSWebCoreIDEOptions, FraWebCoreOptions;

const
  sTMSWebCoreApp = 'TMS Web Core Application';
  sTMSWebCoreAppDescription = 'A Web application using the TMS Web Core technology';

type

  // Project descriptor for IDE item "TMS Web Core application".
  { TProjectWebcoreApp }

  TProjectWebcoreApp = class(TProjectDescriptor)
  private
    FDoNotCreateForm: boolean;
    FRunOnReady: boolean;
    // add a default HTML file to the project.
    function CreateHTMLFile(AProject: TLazProject; {%H-}AFileName: String): TLazProjectFile;
    // Create the project file sources.
    function CreateProjectSource: String;
  protected
    function ShowOptionsDialog: TModalResult; virtual;
    function CheckWebCorePackageLink: boolean; virtual;
    function CheckPackageExists(PkgName,DefaultPkgFilename: string): boolean; virtual;
    function PackageFilenameChanged(PkgName,DefaultPkgFilename: string): boolean; virtual;
  public
    constructor Create; override;
    function DoInitDescriptor : TModalResult; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
    function InitProject(AProject: TLazProject): TModalResult; override;
    function CreateStartFiles(AProject: TLazProject): TModalResult; override;
    property RunOnReady : boolean Read FRunOnReady Write FRunOnReady;
    property DoNotCreateForm : boolean Read FDoNotCreateForm Write FDoNotCreateForm;
  end;

  // File descriptor for new form in TMS Web Core application.
  { TWebCoreWebForm }

  TWebCoreWebForm = class(TFileDescPascalUnitWithResource)
  private
    function CreateWebForm(const Filename, {%H-}SourceName, {%H-}ResourceName: string
      ): string;
  public
    constructor Create; override;
    function CreateSource(const Filename, SourceName,
                          ResourceName: string): string; override;
    function GetInterfaceUsesSection: string; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
    function GetInterfaceSource(const {%H-}Filename, {%H-}SourceName,
                                {%H-}ResourceName: string): string;override;
    function GetImplementationSource(const Filename, SourceName,
                                ResourceName: string): string; override;
  end;

  // File descriptor for new web datamodule in TMS Web Core application.
  { TWebCoreWebDatamodule }

  TWebCoreWebDatamodule = class(TFileDescPascalUnitWithResource)
  public
    constructor Create; override;
    function CreateSource(const Filename, SourceName,
                          ResourceName: string): string; override;
    function GetInterfaceUsesSection: string; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
    function GetInterfaceSource(const {%H-}Filename, {%H-}SourceName,
                                {%H-}ResourceName: string): string;override;
    function GetImplementationSource(const Filename, SourceName,
                                ResourceName: string): string; override;
  end;

var
  WebCoreOptionsFrameID : integer = 10000;

Const
  // Position in project options dialog.
  WebCoreOptionsIndex  = ProjectOptionsMisc + 1000;
  // Name of WebForm file descriptor
  STMSWebCoreForm = 'TMSWebCoreWebForm';
  STMSWebCoreDatamodule = 'TMSWebCoreWebDataModule';

procedure Register;

implementation

uses
  frawebcoreprojectoptions,
  frmnewwebcoreprojectoptions;

Type

  { TWebDataModule }

  TWebDataModule = Class(TDatamodule)
  Private
    FDSIze : TPoint;
    procedure ReadH(Reader: TReader);
    procedure ReadW(Reader: TReader);
    procedure WriteH(Writer: TWriter);
    procedure WriteW(Writer: TWriter);
  Protected
    // We must override this and NOT write the properties that FPC usually writes:
    // PPI and horizontaloffset/verticaloffset
    // Because the TMS DFM-to-Pascal converter does not yet handle them.
    // Only width and height are written.
    // When the TMS DFM-to-Pascal converter does handle these properties, the override can be removed.
    Procedure DefineProperties(Filer: TFiler); override;
  end;

procedure Register;

begin
  // done by WEBLib.Reg : WebFormClass:=TWebForm;
  WebDataModuleClass:=TWebDataModule;
  WebCoreOptions:=TTMSWebcoreIDEOptions.Create;
  WebCoreOptions.Load;
  IDEMacros.Add(WebCoreOptions.TMSWebDirMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreDefaultURLMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreURLMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreCompilerMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreCompilerOptsMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreHTMLMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreRunnerMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreBrowserMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreServerPortMacro);
  IDEMacros.Add(WebCoreOptions.WebCoreWebRunnerIniMacro);

  // register new - project item
  RegisterProjectDescriptor(TProjectWebcoreApp.Create);
  // Register New - Module - Web Core Form file item.
  RegisterProjectFileDescriptor(TWebCoreWebForm.Create);
  RegisterProjectFileDescriptor(TWebCoreWebDataModule.Create);
  // add IDE options frame
  WebCoreOptionsFrameID:=RegisterIDEOptionsEditor(GroupEnvironment,TWebCoreOptionsFrame, WebCoreOptionsIndex)^.Index;
  // Add project options frame
  RegisterIDEOptionsEditor(GroupProject,TWebCoreProjectOptionsFrame, WebCoreOptionsIndex);
  LazarusIDE.AddHandlerOnProjectBuilding(@WebCoreEventSink.ProjectBuilding,True);
  {$IF Laz_FullVersion>=2010000}
  LazarusIDE.AddHandlerOnRunDebugInit(@WebCoreEventSink.RunDebugInit,True);
  {$ELSE}
  LazarusIDE.AddHandlerOnRunDebug(@WebCoreEventSink.RunDebugInit,True);
  {$ENDIF}
  LazarusIDE.AddHandlerOnRunWithoutDebugInit(@WebCoreEventSink.RunWithoutDebug,True);
  LazarusIDE.AddHandlerOnSaveEditorFile(@WebCoreEventSink.SaveEditorFile,True);
  GlobalDesignHook.AddHandlerAddClicked(@WebCoreEventSink.DesignerAddClicked);
  FormEditingHook.RegisterDesignerBaseClass(WebDataModuleClass);

  WebCoreOptions.FixTMSWebDir;
end;

{ TWebDataModule }

Procedure TwebDataModule.ReadW(Reader: TReader);
begin
  FDSIze.X := Reader.ReadInteger;
end;


Procedure TWebDataModule.WriteW(Writer: TWriter);
begin
  Writer.WriteInteger(FDSIze.X);
end;

Procedure TwebDataModule.ReadH(Reader: TReader);
begin
  FDSIze.Y := Reader.ReadInteger;
end;


Procedure TWebDataModule.WriteH(Writer: TWriter);
begin
  Writer.WriteInteger(FDSIze.Y);
end;

procedure TWebDataModule.DefineProperties(Filer: TFiler);

var
  Ancestor : TWebDataModule;
  HaveData : Boolean;

begin
  Ancestor := TWebDataModule(Filer.Ancestor);
  HaveData:=(Ancestor=Nil) or
            (FDSize.X<>Ancestor.FDSize.X) or
            (FDSize.Y<>Ancestor.FDSize.Y);
    Filer.DefineProperty('Height', @ReadH, @WriteH, HaveData);
    Filer.DefineProperty('Width', @ReadW, @WriteW, HaveData);
end;

{ TProjectWebcoreApp }

constructor TProjectWebcoreApp.Create;
begin
  inherited Create;
  Name:=sTMSWebCoreApp;
  Flags:=DefaultProjectFlags;
  RunOnReady:=WebCoreDefaultRunOnReady;
end;

function TProjectWebcoreApp.ShowOptionsDialog : TModalResult;
Var
  F : TNewWebCoreProjectOptionsForm;
begin
  // Show dialog to get settings from the user.
  F:=TNewWebCoreProjectOptionsForm.Create(Application);
  try
    Result:=F.ShowModal;
    if Result=mrOK then
      begin
      Self.RunOnReady:=F.RunOnReady;
      // Gimmick.
      Self.DoNotCreateForm:=F.DoNotCreateForm;
      end;
  finally
    F.Free;
  end;
end;

function TProjectWebcoreApp.CheckWebCorePackageLink: boolean;
var
  TMSWebDir, NewTMSWebDir: String;
  MsgResult: Integer;
begin
  Result:=false;
  TMSWebDir:=ChompPathDelim(WebCoreOptions.GetParsedOptionValue(wccoTMSWebDir));

  if TMSWebDir='' then
    begin
    IDEMessageDialog('Error',
      'TMSWebDir is not set.'+sLineBreak
      +'See Tools / Options / Environment / TMS Web Core',
      mtError,[mbOk]);
    exit;
    end;
  if not DirPathExists(TMSWebDir) then
    begin
    IDEMessageDialog('Error',
      'TMSWebDir is missing "'+TMSWebDir+'".'+sLineBreak
      +'See Tools / Options / Environment / TMS Web Core',
      mtError,[mbOk]);
    exit;
    end;

  if not WebCoreOptions.CheckTMSWebDir(TMSWebDir) then exit;

  if (laz_fullversion<2000100) and (Pos(' ',TMSWebDir)>0)
      and WebCoreOptions.TMSWebDirAskRename then
    begin
    // Lazarus 2.0.0 did not support spaces in compiler path
    NewTMSWebDir:=ExtractFilePath(TMSWebDir)+ReplaceStr(ExtractFileName(TMSWebDir),' ','');
    if Pos(' ',NewTMSWebDir)>0 then
      begin
      MsgResult:=IDEMessageDialog('Warning',
        'The TMS pas2js.exe is installed in a path with spaces, which is not supported by Lazarus 2.0.0.'+sLineBreak
        +'Please rename the directory "'+TMSWebDir+'" or move it to another location.',
        mtWarning,[mbCancel,mbIgnore]);
      if MsgResult<>mrIgnore then exit(false);
      end
    else
      begin
      MsgResult:=IDEMessageDialog('Warning',
        'The TMS WebCore is installed in a path with spaces, which is not supported by Lazarus 2.0.0.'+sLineBreak
        +'Rename the directory to'+sLineBreak
        +NewTMSWebDir+' ?',
        mtWarning,[mbYes,mbNo,mbCancel]);
      case MsgResult of
      mrNo:
        begin
        WebCoreOptions.TMSWebDirAskRename:=false;
        WebCoreOptions.Save;
        end;
      mrYes:
        repeat
          if DirPathExists(NewTMSWebDir)
              or RenameFileUTF8(TMSWebDir,NewTMSWebDir) then
            begin
            InvalidateFileStateCache;
            TMSWebDir:=NewTMSWebDir;
            WebCoreOptions.TMSWebDir:=TMSWebDir;
            // reopen packages
            if not PackageFilenameChanged(TMSWEBCoreRTL_PkgName,TMSWEBCoreRTL_PkgFile) then exit;
            if not PackageFilenameChanged(TMSWEBCore_PkgName,TMSWEBCore_PkgFile) then exit;
            if not PackageFilenameChanged(TMSWEBCoreDe_PkgName,TMSWEBCoreDe_PkgFile) then exit;
            if not PackageFilenameChanged(TMSWEBCoreLib_PkgName,TMSWEBCoreLib_PkgFile) then exit;
            break;
            end;
          MsgResult:=IDEMessageDialog('Error',
            'Unable to rename'+sLineBreak
            +TMSWebDir+sLineBreak
            +'to '+sLineBreak
            +NewTMSWebDir+sLineBreak
            +'Close any program locking files there, e.g. browser and TMSWebServer.',
            mtError,[mbCancel,mbIgnore,mbRetry]);
          case MsgResult of
          mrRetry: ;
          mrIgnore: break;
          else exit(false);
          end;
        until false;
      else
        exit(false);
      end;
      end;
    end;

  // check that all needed packages are in place
  // For example after an update or if user moved directories some lpk might need a reload

  if not CheckPackageExists(TMSWEBCoreRTL_PkgName,TMSWEBCoreRTL_PkgFile) then exit;
  if not CheckPackageExists(TMSWEBCore_PkgName,TMSWEBCore_PkgFile) then exit;
  if not CheckPackageExists(TMSWEBCoreDe_PkgName,TMSWEBCoreDe_PkgFile) then exit;
  if not CheckPackageExists(TMSWEBCoreLib_PkgName,TMSWEBCoreLib_PkgFile) then exit;
  // ToDo: check version
  Result:=true;
end;

function TProjectWebcoreApp.CheckPackageExists(PkgName, DefaultPkgFilename: string
  ): boolean;
var
  Pkg: TIDEPackage;
  LPKFilename, TMSWebDir: String;
  PkgLink: TPackageLink;
begin
  Pkg:=PackageEditingInterface.FindPackageWithName(PkgName);
  if Pkg<>nil then
    begin
    LPKFilename:=Pkg.Filename;
    if FileExistsUTF8(LPKFilename) then
      exit(true);
    Result:=PackageFilenameChanged(PkgName,DefaultPkgFilename);
    end
  else
    begin
    // package not open -> check links
    Result:=false;
    PkgLink:=PkgLinks.FindLinkWithPkgName(PkgName);
    if PkgLink<>nil then
      begin
      if FileExistsUTF8(PkgLink.GetEffectiveFilename) then
        exit(true);
      end;
    // add link
    TMSWebDir:=WebCoreOptions.GetParsedOptionValue(wccoTMSWebDir);
    LPKFilename:=AppendPathDelim(TMSWebDir)+DefaultPkgFilename;
    if not FileExistsUTF8(LPKFilename) then
      begin
      IDEMessageDialog(TMSWEBCore_PkgName+'.lpk not found',
        'File not found: "'+LPKFilename+'"'+sLineBreak
        +'Check Tools / Options / Environment / TMS Web Core.',
        mtError,[mbOk]);
      exit;
      end;

    PkgLink:=PkgLinks.AddUserLink(LPKFilename,PkgName);
    if PkgLink=nil then
      begin
      IDEMessageDialog('Failed adding lpk',
        'Failed to add package link "'+LPKFilename+'"',
        mtError,[mbOk]);
      exit;
      end;

    Result:=true;
    end;
end;

function TProjectWebcoreApp.PackageFilenameChanged(PkgName, DefaultPkgFilename: string
  ): boolean;
var
  Pkg: TIDEPackage;
  LPKFilename: String;
  MsgResult: Integer;
begin
  Pkg:=PackageEditingInterface.FindPackageWithName(PkgName);
  if Pkg=nil then exit(true);
  LPKFilename:=AppendPathDelim(WebCoreOptions.GetParsedOptionValue(wccoTMSWebDir))+DefaultPkgFilename;
  repeat
    if FileExistsUTF8(LPKFilename) then
      break;
    MsgResult:=IDEMessageDialog('Error',
          'Package "'+PkgName+'" not found:'+sLineBreak
          +LPKFilename,
          mtError,[mbCancel,mbRetry,mbIgnore]);
    case MsgResult of
    mrIgnore: exit(true);
    mrRetry:
      begin
      InvalidateFileStateCache;
      continue;
      end;
    else
      exit(false);
    end;
  until false;
  if Pkg.Filename=LPKFilename then
    exit(true);
  Result:=PackageEditingInterface.DoOpenPackageFile(LPKFilename,
                                [pofAddToRecent,pofDoNotOpenEditor],false)=mrOk;
end;

function TProjectWebcoreApp.DoInitDescriptor: TModalResult;
begin
  Result:=ShowOptionsDialog;
  if Result=mrOk then
    begin
    // make sure package webcore can be found
    if not CheckWebCorePackageLink then
      exit(mrCancel);
    end;
end;

function TProjectWebcoreApp.GetLocalizedName: string;
begin
  Result:=sTMSWebCoreApp;
end;

function TProjectWebcoreApp.GetLocalizedDescription: string;
begin
  Result:=sTMSWebCoreAppDescription;
end;

function TProjectWebcoreApp.CreateHTMLFile(AProject: TLazProject;
  AFileName: String): TLazProjectFile;
Const
  TemplateHTMLSource =
     '<!doctype html>'+LineEnding
    +'<html lang="en">'+LineEnding
    +'<head>'+LineEnding
    +'  <meta charset="utf-8">'+LineEnding
    +'  <meta name="viewport" content="width=device-width, initial-scale=1">'+LineEnding
    +'  <meta $(ThemeColor)>'+LineEnding
    +'  <noscript>Your browser does not support JavaScript!</noscript>'+LineEnding
    +'  <link rel="icon" href="data:;base64,=">'+LineEnding
    +'  <title>TMS Web Project</title>'+LineEnding
    +'  <script type="text/javascript" src="$(ProjectName).js"></script>'+LineEnding
    +'</head>'+LineEnding
    +'<body>'+LineEnding
    +'  <script>'+LineEnding
    +'  %s'+LineEnding
    +'  </script>'+LineEnding
    +'</body>'+LineEnding
    +'</html>'+LineEnding;

Var
  HTMLFile : TLazProjectFile;
  HTMLSource : String;
  RunScript : String;
begin
  HTMLFile:=AProject.CreateProjectFile('project1.html');
  HTMLFile.IsPartOfProject:=true;
  HTMLFile.CustomData.Values[KeyIsProjectHTMLFile]:='1';
  AProject.AddFile(HTMLFile,false);
  if RunOnReady then
    RunScript:='window.addEventListener("load", rtl.run);'+LineEnding
  else
     RunScript:='rtl.run();'+LineEnding;
  HTMLSource:=Format(TemplateHTMLSource,[RunScript]);
  HTMLFile.SetSourceText(HTMLSource);
  Result:=HTMLFile;
end;

function TProjectWebcoreApp.CreateProjectSource : String;
Var
  Src : TStrings;
  units : string;

  Procedure Add(aLine : String);
  begin
    Src.Add(aLine);
  end;

  Procedure AddLn(aLine : String);
  begin
    if (Aline<>'') then
      Aline:=Aline+';';
    Add(Aline);
  end;

begin
  // Initial uses clause for project. Add units as appropriate...
  Units:='  Forms';
  Src:=TStringList.Create;
  try
    // create default program source. Add lines as needed.
    AddLn('program Project1');
    AddLn('');
    Add('{$MODE Delphi}');
    Add('');
    Add('uses');
    AddLn(units+'');
    Add('');
    Add('{$R *.res}');
    Add('');
    Add('begin');
    AddLn('  Application.Initialize');
    AddLn('  Application.MainFormOnTaskBar:=True');
    AddLn('  Application.Run');
    Add('end.');
    Result:=Src.Text;
  finally
    Src.Free;
  end;
end;

function TProjectWebcoreApp.InitProject(AProject: TLazProject): TModalResult;
var
  MainFile,
  HTMLFile : TLazProjectFile;
  CompOpts: TLazCompilerOptions;
begin
  Result:=inherited InitProject(AProject);
  // The following can also be done in startproject...
  MainFile:=AProject.CreateProjectFile('project1.lpr');
  MainFile.IsPartOfProject:=true;
  AProject.AddFile(MainFile,false);
  AProject.MainFileID:=0;
  CompOpts:=AProject.LazCompilerOptions;
  SetDefaultWebCoreCompileOptions(CompOpts);
  CompOpts.TargetFilename:=AppendPathDelim(WebCoreDefaultOutputPath)+'project1.js';
  CompOpts.TargetFilenameApplyConventions:=False;
  SetDefaultWebCoreProjectRunParams(AProject);
  AProject.MainFile.SetSourceText(CreateProjectSource,true);
  // Mark this project as a TMS Web Core project.
  AProject.CustomData.Values[KeyTMSWebCoreProject]:='1';
  // create default project HTML file.
  HTMLFile:=CreateHTMLFile(aProject,'project1.js');
  // Mark it so we know that this is the project HTML file.
  HTMLFIle.CustomData[KeyIsProjectHTMLFile]:='1';
  // Let's remember this setting in case we need to regenerate the HTML
  if RunOnReady then
    AProject.CustomData[KeyProjectRunOnReady]:='1';
  // Add dependency on TMSWebcore. Add more calls with other packages as needed.
  AProject.AddPackageDependency('TMSWebCore');
  debugln(['Info: (lazarus) [TProjectWebcoreApp.InitProject] CompilerPath=[',AProject.LazCompilerOptions.CompilerPath,']']);
end;

function TProjectWebcoreApp.CreateStartFiles(AProject: TLazProject
  ): TModalResult;
begin
  Result:=LazarusIDE.DoOpenEditorFile(AProject.MainFile.Filename,-1,-1,
                                      [ofProjectLoading,ofRegularFile]);
  if Result<>mrOK then
     exit;
  Result:=LazarusIDE.DoOpenEditorFile('project1.html',-1,-1,
                                     [ofProjectLoading,ofRegularFile]);
  if Result<>mrOK then
     exit;
  if not DoNotCreateForm then
    Result:=LazarusIDE.DoNewEditorFile(ProjectFileDescriptors.FindByName(STMSWebCoreForm),'','',
                                       [nfIsPartOfProject,nfOpenInEditor,nfCreateDefaultSrc]);
end;

{ TWebCoreWebForm }

constructor TWebCoreWebForm.Create;
begin
  inherited Create;
  Name:=STMSWebCoreForm; // 'TMSWebCoreWebForm';
  DefaultFilename:='unit1.pas';
  DefaultSourceName:='unit1';
  ResourceClass:=WebFormClass;
  DefaultResourceName:='Form1';
  UseCreateFormStatements:=True;
  RequiredPackages:='TMSWebCore';
end;

function TWebCoreWebForm.CreateWebForm(const Filename, SourceName, ResourceName: string): string;
begin
  With TStringList.Create do
   try
     Add('<html>');
     Add('  <head>');
     Add('    <meta http-equiv="Content-type" content="text/html; charset=utf-8" />');
     Add('    <title>TMS Web Project</title>');
     Add('    <style>');
     Add('    </style>');
     Add('  </head>');
     Add('  <body>');
     Add('  </body>');
     Add('</html>');
     Result:=Text;
     if ExtractFilePath(FileName)='' then
       SaveToFile(GetTempDir(False)+ChangeFileExt(ExtractFileName(FileName),'.html'))
     else
       SaveToFile(ChangeFileExt(FileName,'.html'));
     // Todo: register in IDE ?
   finally
     Free;
   end;
end;

function TWebCoreWebForm.CreateSource(const Filename,
  SourceName, ResourceName: string): string;
var
  LE: string;
begin
  if (LazarusIDE.ActiveProject=nil)
      or (LazarusIDE.ActiveProject.CustomData[KeyTMSWebCoreProject]<>'1') then
     Raise Exception.Create('Project is not a TMS Web Project');
  CreateWebForm(FileName,SourceName,ResourceName);
  // Optionally show a dialog to get options here...
  LE:=LineEnding;
  Result:=
     'unit '+SourceName+';'+LE
    +LE
    +'{$mode delphi}'+LE
    +LE
    +'interface'+LE
    +LE
    +'uses'+LE
    +'  '+GetInterfaceUsesSection+';'+LE
    +LE
    +GetInterfaceSource(Filename,SourceName,ResourceName)
    +'implementation'+LE
    +LE
    +GetImplementationSource(Filename,SourceName,ResourceName)
    +'end.'+LE
    +LE;
end;

function TWebCoreWebForm.GetInterfaceUsesSection: string;
begin
  Result:=inherited GetInterfaceUsesSection;
  // Adjust as needed.
  Result:='SysUtils, Classes, WEBLib.Forms, WEBLib.Graphics, WEBLib.Controls, WEBLib.Dialogs';
end;

function TWebCoreWebForm.GetLocalizedName: string;
begin
  // Name in the 'File - New' items tree. Adjust as needed.
  Result:='TMS Web Core Form';
end;

function TWebCoreWebForm.GetLocalizedDescription: string;
begin
  // Shown in the right-hand panel when the item is selected in the 'File - New' items tree.
  // Adjust as needed.
  Result:='a TMS Web core Form for a TMS Web Core Web application';
end;

function TWebCoreWebForm.GetInterfaceSource(const Filename,
  SourceName, ResourceName: string): string;

begin
  // Inherited method is doing just fine. Adjust if needed.
  Result:=Inherited GetInterfaceSource(FileName,SourceName,ResourceName);
end;

function TWebCoreWebForm.GetImplementationSource(
  const Filename, SourceName, ResourceName: string): string;
begin
  // Inherited method is doing just fine. Adjust if needed.
  Result:=Inherited GetImplementationSource(FileName,SourceName,ResourceName);
end;

{ TWebCoreWebDataModule }

constructor TWebCoreWebDataModule.Create;
begin
  inherited Create;
  Name:=STMSWebCoreDatamodule ; // 'TMSWebCoreWebDatamodule';
  DefaultFilename:='unit1.pas';
  DefaultSourceName:='unit1';
  ResourceClass:=WebDataModuleClass;
  DefaultResourceName:='WebDataModule1';
  UseCreateFormStatements:=True;
  RequiredPackages:='TMSWebCore';
end;


function TWebCoreWebDataModule.CreateSource(const Filename,
  SourceName, ResourceName: string): string;
var
  LE: string;
begin
  if (LazarusIDE.ActiveProject=nil)
      or (LazarusIDE.ActiveProject.CustomData[KeyTMSWebCoreProject]<>'1') then
     Raise Exception.Create('Project is not a TMS Web Project');
  // Optionally show a dialog to get options here...
  LE:=LineEnding;
  Result:=
     'unit '+SourceName+';'+LE
    +LE
    +'{$mode delphi}'+LE
    +LE
    +'interface'+LE
    +LE
    +'uses'+LE
    +'  '+GetInterfaceUsesSection+';'+LE
    +LE
    +GetInterfaceSource(Filename,SourceName,ResourceName)
    +'implementation'+LE
    +LE
    +GetImplementationSource(Filename,SourceName,ResourceName)
    +'end.'+LE
    +LE;
end;

function TWebCoreWebDataModule.GetInterfaceUsesSection: string;
begin
  // Adjust as needed.
  Result:='SysUtils, Classes, WEBLib.Modules';
end;

function TWebCoreWebDataModule.GetLocalizedName: string;
begin
  // Name in the 'File - New' items tree. Adjust as needed.
  Result:='TMS Web Core Datamodule';
end;

function TWebCoreWebDataModule.GetLocalizedDescription: string;
begin
  // Shown in the right-hand panel when the item is selected in the 'File - New' items tree.
  // Adjust as needed.
  Result:='a TMS Web core Data Module for a TMS Web Core Web application';
end;

function TWebCoreWebDataModule.GetInterfaceSource(const Filename,
  SourceName, ResourceName: string): string;

begin
  // Inherited method is doing just fine. Adjust if needed.
  Result:=Inherited GetInterfaceSource(FileName,SourceName,ResourceName);
end;

function TWebCoreWebDataModule.GetImplementationSource(
  const Filename, SourceName, ResourceName: string): string;
begin
  // Inherited method is doing just fine. Adjust if needed.
  Result:=Inherited GetImplementationSource(FileName,SourceName,ResourceName);
end;

end.
